/*
 * I6400 I6500/set_gpr_boot_values.S
 *
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include <boot_64.h>
#include <mips/asm.h>
#include <mips/m64c0.h>
#include <mips/regdef.h>
#include <cm3.h>

/**************************************************************************************
**************************************************************************************/
LEAF(set_gpr_boot_values)

    mfc0    a0, C0_EBASE                		// Read CP0 EBase
	ext	    r23_cpu_num, a0, 0, 10     			// Extract core Number (VP number in CPS)
	ext	    r9_vpe_num, a0,  0, 2				// Extract VP Number
	dla 	r24_boston_byte, boston_lcd_display_byte // load address of display function
    dla     gp, _gp                			// All vpe share globals.
    li      sp, STACK_BASE_ADDR     			// Each CPU gets it's own stack.
    ins     sp, r23_cpu_num, STACK_SIZE_LOG2, 5

check_cps: // Determine if there is a coherency manager present. (Implementation Dependent.)

    mfc0    r25_coreid, C0_PRID					// CP0 PRId.
    ext     a0, r25_coreid, 8, 16				// Extract Manuf and Core.
    li      a3, 0x01b0							// I6500 Family Core ID
    beq     a3, a0, is_cps

is_not_I6500: // This processor is not an I6500 Core so exit!
	sdbbp										// Failed assertion: not I6400.

is_cps:
	li      r11_is_cps, 1

    // Verify that we can find the GCRs.
	dla	    a1, GCR_CONFIG_ADDR					// KSEG1(GCRBASE)
	ld	    a0, GCR_BASE(a1)					// read GCR_BASE
    ins     a1, $0, 29, 3 						// Convert KSEG1 to physical address.
	beq	    a1, a0, gcr_found
    sdbbp   // Can't find GCR. RTL config override of MIPS default?

gcr_found:
    // Every vpe will set up the following to simplify resource initialization.
    li      r22_gcr_addr, GCR_CONFIG_ADDR

    ld      r8_core_num, (CORE_LOCAL_CONTROL_BLOCK + GCR_CL_ID) (r22_gcr_addr) // Load GCR_CL_ID
    ld      a0, GCR_CONFIG(r22_gcr_addr)      	// Load GCR_CONFIG

	ext	    r19_more_cores, a0, PCORES_SHIFT, PCORES_BITS  // Extract # of Physical Cores 0 is 1 core, 1 is 2 ...

	ld      a0, (CORE_LOCAL_CONTROL_BLOCK + GCR_CL_CONFIG) (r22_gcr_addr) // Load GCR_CL_CONFIG
	dext	r20_more_vpes, a0, PVPE_SHIFT, PVPE_BITS		// Extract # of VPs in Core 0 is 1 core, 1 is 2 ...

    dli     a0, CPC_P_BASE_ADDR					// Locate CPC in memory map
    sd      a0, GCR_CPC_BASE(r22_gcr_addr)		// Write CPC_BASE address to GCR
    dli     r30_cpc_addr, CPC_BASE_ADDR			// Maintain address of CPC register block.

done_set_gpr:
    jalr	zero,     ra
END(set_gpr_boot_values)
