/*
 * I6400 I6500/start.S
 *
 *  Start of boot code
Copyright (c) 2007-2019, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#include <boot_64.h>
#include <mips/asm.h>
#include <mips/m64c0.h>
#include <mips/regdef.h>
#include <cm3.h>

//*************************************************************************************
//    R E S E T   E X C E P T I O N   H A N D L E R
//*************************************************************************************

LEAF(__reset_vector)
    dmtc0	zero, C0_COUNT  	    // Clear cp0 Count (Used to measure boot time.)
    b	1f

	// Note: Address 0x1FC0.0010 is "special", in the sense that it is overridden
	// and does not decode to an address in the SW-EPROM, but rather to memory
	// mapped register which holds the board REVISION number. This makes the
	// following nops necessary to insure the next code segment does not fall
	// into the Revision address range
	nop
	nop
	nop
1:
	dla a2, __cpu_init			// NOTE: intrinsic instruction will result in
								// 6 actual instructions.
	jr	a2

END(__reset_vector)

#ifdef RTEST
start_test:  .WORD     0:8
#endif

//*************************************************************************************
//    B O O T   E X C E P T I O N   H A N D L E R S
//*************************************************************************************

.org 0x200 // TLB refill.
#ifdef RTEST
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait
#else
	addu a0,  zero, r23_cpu_num
	li a1, 0x58 // X - Show  exception
	jalr r24_boston_byte

	sdbbp
1:  b       1b  // Stay here
#endif

.org 0x280 // XTLB refill, 64 bit task.
#ifdef RTEST
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait
#else
	addu a0,  zero, r23_cpu_num
	li a1, 0x58 // X - Show  exception
	jalr r24_boston_byte

	sdbbp
1:  b       1b  // Stay here
#endif

.org 0x300 // Cache error exception.
#ifdef RTEST
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait
#else
	addu a0,  zero, r23_cpu_num
	li a1, 0x58 // X - Show  exception
	jalr r24_boston_byte

	sdbbp
1:  b       1b  // Stay here
#endif

.org 0x380 // General exception.
	// expecting an inter-processor interrupt
	addu a0,  zero, r23_cpu_num
	li a1, 0x58 // X - Show  exception
	jalr r24_boston_byte

#ifdef RTEST
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait
#else
	sdbbp
1:  b       1b  // Stay here
#endif

.org 0x480 // EJTAG debug exception (EJTAG Control Register[ProbTrap] == 0.)

#ifdef RTEST
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait
#else
	addu a0,  zero, r23_cpu_num
	li a1, 0x58 // X - Show  exception
	jalr r24_boston_byte

1:  b       1b  // Stay here
#endif

.org 0x500 // Resume code past the boot exception vectors.
//*************************************************************************************
//*************************************************************************************
LEAF(__cpu_init)

check_nmi: // Verify we are here due to a reset (and not NMI.)
    mfc0    a0, C0_STATUS				// Read CP0 Status
    ins		a0, zero, 1, 1				// clear EXL so C0_EPC will be set on exception
    mtc0	a0, C0_STATUS				// Write back C0 Status
    ext		a0, a0, 19, 1				// Extract NMI bit
    beqz    a0, verify_isa				// Branch if this is NOT an NMI exception.

    addu	a0,  zero, r23_cpu_num
	li		a1, 0x4e // N - Show  NMI
	jalr	r24_boston_byte
1:
	b	1b
    sdbbp								// NMI.


verify_isa: // Verify device ISA meets code requirements (MIPS64R6 or later.)

#ifndef RTEST
	#ifdef BOSTON_RAM  // make sure ram is up before checking array
		dla 	a2, init_FPGA_mem		// Initialize the ROC-it2 MC (Memory Controller.)
		jalr 	a2
	#endif
	// Cores and VPs must be started in sequence
	// There is a word size element for each VP in the
	// cluster. This element is set in power_up_cores or
	// start_vps
    dla     k0, go_vp
	li		k1, 0xA0000000				// make uncached
	or		k0, k0, k1
    mfc0    k1, C0_EBASE				// Get cp0 EBase
    dext    k1, k1, 0, 10				// Extract CPUNum
    beqz	k1, 1f						// C0V0 needs to continue
    dsll    k1, k1, 2					// x 4 for integer element
    daddu   k0, k0, k1					// index into go_vp array
    dli		a0, 1
wait_for_go:
    lw		k1, 0(k0)
    beq		k1, a0, 1f					// Is the go bit for this VP set?
    wait								// This will cause the core to idle while waiting to be reset in
    									// power_up_cores or start_vps
    b wait_for_go						// retest if interrupted (debugger can interrupt)
1:
#endif
    mfc0    a0, C0_CONFIG				// Read CP0 Config
    ext	    a1, a0, 13, 2     			// Extract Architecture Type (AT)
    li		a3, 2						// load expected value for MIP642R6 ISA and version
    beq		a3, a1, is_mips64			// Branch if executing on MIP642R6 ISA.
    sdbbp 								// Failed assertion: MIPS64R6.

is_mips64:
    ext	    a2, a0, 10, 2     			// Extract Architecture Release (AR)
    beq		a3, a2, init_common_resources	// Continue if ISA is MIPS64R6 or later.
    sdbbp								// Failed assertion MIPS64R6.


//*************************************************************************************
// What is initialized on execution depends on the core and VP executing it.
// *************************************************************************************

// All VPs
init_common_resources:  				// initializes resources for virtual processor.

    dla 	a2,init_gpr					// Fill register file with dummy value then boot info.
	jalr 	a2

    dla 	a2,	set_gpr_boot_values  // Fill register file boot info.
	jalr 	a2

    dla 	a2, init_cp0 				// Init CP0 Status, Count, Compare, Watch*, and Cause.
    jalr 	a2

    beqz    r23_cpu_num, C0V0			// If Core 0 VP 0 skip other core continue loop

#ifndef SIM
wait_for_core0_vp0:
    // Check to see if Core 0 VP 0 is past the point where other cores can continue
    addu a0,  zero, r23_cpu_num			// set argument a0 to cpu number
    li a1, 0x77 						// w - Show that code is in wait loop
    jalr r24_boston_byte

	// Check Core0 VP 0 GCR Other Coherency register to see if it got to the
	// join_domain point so this core can continue
	sw      zero, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_OTHER)(r22_gcr_addr) // GCR_CL_OTHER[Core0 VP0]
    lw      a0, (CORE_OTHER_CONTROL_BLOCK | GCR_CO_COH_EN)(r22_gcr_addr)  // GCR_CO_COHERENCE
    beqz    a0, wait_for_core0_vp0   	// Busy wait on core 0 vp0 to join_domain.

#endif
    b		init_core_resources

C0V0:
// The L2 cache needs to be disabled for now, until we check if it has been
// initialized by hardware (HCI), or until we initialize it with software.
// Disable the L2 cache by setting the L2_Bypass bit in the CM3 L2 Configuration register.
	ld		a0, GCR_L2_CONFIG(r22_gcr_addr)	// Read L2 Configuration register
	dli		a3, 0x1								// set to uncached (bypass)
	dins	a0, a3, 20, 1    					// Insert bits
	sd		a0, GCR_L2_CONFIG(r22_gcr_addr)	// Write L2 Configuration register
	sync

init_core_resources:

// All VPs will have a VTLB executing a TLBINVF with Index in VTLB range,
// 0 in this case, causes all entries in the VTLB to be invalidated.
	mtc0	zero, C0_INDEX
	tlbinvf								// invalidate all VTLB entries

// Only VP0 on each core should initialize the instruction cache and TLB and FTLB entries
    bnez    r9_vpe_num, cache_on

    dla 	a2, init_ftlb    			// Invalidate all FTLB entries.
    jalr 	a2

    dla 	a2, init_icache 			// Initialize the L1 instruction cache. (Executing using I$ on return.)
    jalr 	a2

cache_on:
// All VPs must turn cache on.
// You should not do any cacheable data accesses (load/store/pref) before the L2 and
// coherence are fully enabled.  You can however do CCA5 instruction fetches while
// the L2 is in bypass mode (so the SW cache initialization routine can be cached in the L1 Icache).

// Once the Icache has been turned on, instruction fetches from KSEG0 will be cached.
// This makes initializing the Dcache and the L2 cache much faster since the
// code will be cached.

// The changing of Kernel mode cacheability (Config.K0) must be done from KSEG1.
// Since this code is executing from KSEG0, it needs to do a jump to KSEG1,
// change K0, and then jump back to KSEG0.

	dla 	a2,	change_k0_cca
	dli     a1, 0xf
	ins    	a2, a1, 29, 1 				// changed to KSEG1 address by setting bit 29
    jalr 	a2

// Only VP0 on each core should initialize the data cache
    bnez    r9_vpe_num, init_sys_resources_done

    dla		a2, init_dcache 			// Initialize the L1 data cache
    jalr 	a2

    bnez    r8_core_num, init_sys_resources_done // Only core0/vp0 needs to init system resources.

// 	Do System level initialization only once for the CPS (Core 0, VP 0)

    dla 	a2, init_L2   				// Initialize the L2 cache
    jalr 	a2

    dla 	a2, init_cm     			// Initialize the CPS CM ( Coherency Manager.)
    jalr 	a2

#ifndef RTEST
    dla 	a2, copy_c2_ram 			// Copy "C" code and data to RAM and zero bss (uncached.)
    jalr 	a2
#endif

    dla 	a2, power_up_cores  		// Power up other cores to execute this boot code.
    jalr 	a2

init_sys_resources_done:


    bnez    r9_vpe_num, init_done       // Only vp0 of each core need to start other VPs and join coherency domain.

    dla 	a2, start_vps 				// each core(only VP 0) needs to start other VPs
    jalr 	a2

    dla 	a2, join_domain 			// each core(only VP 0) needs to enable Coherence
    jalr 	a2

init_done:

	// enable 64bit access
	li		a2, SR_KX | SR_SX | SR_UX
	mfc0	a1, C0_STATUS
	or		a1, a1, a2
	mtc0	a1, C0_STATUS

    // Prepare for eret to main (sp and gp set up per vpe in set_gpr_boot_values).
    dla      ra, all_done				// If main returns then go to all_done:.
    dla      a1, main
    dmtc0    a1, C0_ERRPC 				// ErrorEPC

    // Prepare arguments for main()
    daddu    a0, zero, r23_cpu_num  	// main(arg0) is the "cpu" number,cp0 EBase[CPUNUM]
    daddiu   a1, r8_core_num, 0			// main(arg1) is the core number.
    daddiu   a2, r9_vpe_num, 0			// main(arg2) is the vpe number.
    daddiu   a3, r20_more_vpes, 1		// main(arg3) is the number of vpe on this core.
    eret    // Exit reset exception handler for this CPU and start execution of main().

//*************************************************************************************
//*************************************************************************************
all_done:
    // Looks like main returned. Just busy wait spin.
    b       all_done

END(__cpu_init)
