
/***********************************************************************
 *
 *  sead_SPRam.ld for m14Kc
 *
 *  Linker script for startup code for Sead board 
 *
 * ######################################################################

Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors 

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are 
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of 
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list 
of conditions and the following disclaimer in the documentation and/or other materials 
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be 
used to endorse or promote products derived from this software without specific prior 
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


OUTPUT_ARCH(mips)

/**** Start point ****/
ENTRY(__reset_vector)

SECTIONS
{
  /* _FMT_offset is used to translat from the physical address of the SP Rams to the */
  /* virtual address the code and data need to be linked at */
  _FMT_offset  = 0x40000000 ;
  _monitor_flash = 0x1fc00000 ;
  _ISPram = 0x50000000 ;
  _DSPram = 0x60000000 ;
  _code_start = _ISPram - _FMT_offset ;
  _data_start = _DSPram - _FMT_offset ;

  /**** Init Code (executed from reset.) ****/

  .text_init 0x9fc00000 :   /* Runs uncached (from 0x9fc00000) till I$ is initialized. */
  AT( _monitor_flash )      /* (Load address is start of Monitor Flash.) */

  {
    _ftext_init = ABSOLUTE(.) ;	/* Start of init code. */
    start.o(.text)
    ../common/init_gpr.o(.text)
    set_gpr_boot_values.o(.text)
    ../common/init_cp0.o(.text)
    ../common/init_tlb.o(.text)
    ../common/init_caches.o(.text)
    ../common/copy_c2_SPram_MM.o(.text)
    ../sead3/lcd.o(.text)
    . = ALIGN(8);
    _etext_init = ABSOLUTE(.);	/* End of init code. */
  } = 0

  /**** "C" Code and read-only data ****/

  _zap1 = _etext_init - _ftext_init + _monitor_flash;
  _start_rom_text = _etext_init - _ftext_init + 0xbfc00000;

  .text_ram _code_start :    /* Need to copy code and read-only data to ram... */
  AT( _zap1 )               /* (Load address is after init code in Monitor Flash.) */

  {
    _ftext_ram = ABSOLUTE(.) ;	/* Start of code */
    *(.text)*(.text.*)
    . = ALIGN(8);
    _etext_ram = ABSOLUTE(.);	/* End of code  */
  } = 0

  /**** Initialised data ****/

  _zap2 = _etext_ram - _ftext_ram + _zap1 ;

  .data _data_start :
  AT( _zap2 )               /* (Load address is after "C" code in Monitor Flash.) */
  {
    _fdata_ram = ABSOLUTE(.);	/* Start of initialised data	    */
    *(.rodata)
    *(.rodata.*)
    *(.data)
    . = ALIGN(8);
    _gp = ABSOLUTE(. + 0x7ff0); /* Base of small data		    */
    *(.lit8) 
    *(.lit4) 
    *(.sdata) 
    . = ALIGN(8);
    _edata_ram  = ABSOLUTE(.);	/* End of initialised data	    */
  } 
_zap3 = _edata_ram - _fdata_ram + _zap2 ;

  /**** Uninitialised data ****/

  _fbss = .;			/* Start of uninitialised data	    */

  .sbss : 
  { 
    *(.sbss) 
    *(.scommon)
  }
  .bss :
  {
    *(.bss)
    *(COMMON)
  }

  _end = . ;		/* End of uninitialised data	    */

  .gptab.sdata : { *(.gptab.data) *(.gptab.sdata) }
  .gptab.sbss : { *(.gptab.bss) *(.gptab.sbss) }


  /DISCARD/ :
  {
    *(.reginfo)
    *(.MIPS.abiflags)
  }
  _freemem = .;
}
