/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <boot.h>
#include <mips/m32c0.h>
#include <mips/regdef.h>
#include <cps.h>

/**************************************************************************************
**************************************************************************************/
LEAF(set_gpr_boot_values)

    mfc0    a0, C0_EBASE                		// Read CP0 EBase
	ext	    r23_cpu_num, a0, 0, 4     			// Extract CPUNum
    la 		r24_boston_byte, boston_lcd_display_byte // load address of display function
    la      gp, _gp                				// All vpe share globals.
    li      sp, STACK_BASE_ADDR     			// Each CPU gets it's own stack.
    ins     sp, r23_cpu_num, STACK_SIZE_LOG2, 3

no_mt_ase: // This processor does not implement the MIPS32 MT ASE. Set up defaults.
	li 	    r9_vpe_num, 0
	li 	    r18_tc_num, 0
    li 	    r20_more_vpes, 0
    li 	    r21_more_tcs, 0

check_cps: // Determine if there is a coherency manager present. (Implementation Dependent.)
    mfc0    r25_coreid, C0_PRID					// CP0 PRId.
    ext     a0, r25_coreid, 8, 16				// Extract Manuf and Core.
    li      a3, 0x01A8							// P5600 Family Core ID
    beq     a3, a0, is_cps

is_not_ProAptiv: // This processor is not a ProAptive Core so exit!
	sdbbp										// Failed assertion: not ProAptiv.
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

is_cps:
	li      r11_is_cps, 1

    // Verify that we can find the GCRs.
	la	    a1, GCR_CONFIG_ADDR					// KSEG1(GCRBASE)
    ins     a1, $0, 29, 3 						// Convert KSEG1 to physical address.
	lw	    a0, GCR_BASE(a1)					// read GCR_BASE
	ins	    a0, $0, 0, 15						// Isolate physical base address of GCR (Clear lower bits).
	beq	    a1, a0, gcr_found
    sdbbp   // Can't find GCR. RTL config override of MIPS default?
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

gcr_found:
    // Every vpe will set up the following to simplify resource initialization.
    li      r22_gcr_addr, GCR_CONFIG_ADDR
    lw      r8_core_num, (CORE_LOCAL_CONTROL_BLOCK + GCR_CL_ID) (r22_gcr_addr) // Load GCR_CL_ID
    lw      a0, GCR_CONFIG(r22_gcr_addr)      	// Load GCR_CONFIG
	ext	    r19_more_cores, a0, PCORES, PCORES_S // Extract PCORES

done_init_gpr:
    jr     ra
END(set_gpr_boot_values)
