/*
 * P5600/start.S
 *
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <boot.h>
#include <mips/m32c0.h>
#include <mips/regdef.h>
#include <cps.h>

#*************************************************************************************
#    R E S E T   E X C E P T I O N   H A N D L E R
#*************************************************************************************
LEAF(__reset_vector)
    la 	a2, check_nmi
    mtc0	zero, C0_COUNT  	    # Clear cp0 Count (Used to measure boot time.)
    jr	a2

	# Note: Address 0x1FC0.0010 is special, in the sense that it is overridden
	# and does not decode to an address in the SW-EPROM, but rather to memory
	# mapped register which holds the board REVISION number. This makes the
	# following nops necessary to insure the next code segment does not fall
	# into the Revision address range

	nop
	nop
	nop
	nop

END(__reset_vector)

#ifdef RTEST
start_test:  .WORD     0:8
#endif

# Inline the code fill the rest of
# space between here and the next exception vector address.

#*************************************************************************************
#    B O O T   E X C E P T I O N   H A N D L E R S
#*************************************************************************************

.org 0x200 # TLB refill, 32 bit task.
    sdbbp
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

.org 0x280 # XTLB refill, 64 bit task.
    sdbbp
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

.org 0x300 # Cache error exception.
    sdbbp
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

# Inline code to fill the rest of
# space between here and the next exception vector address.

.org 0x380 # General exception.
	// expecting an interprocessor interrupt
	mfc0	k1, C0_CAUSE
	ext		k0, k1, 2, 5				// extract exit code
	bne		k0, zero, other_code		// should be 0 indicating a interrupt
	ext		k0, k1, 10, 5				// extract interrupt
	li		k1, 1
	bne		k0, k1, other_code			// should be interrupt 0

	// interprocessor interrupt
    li      k0, (GIC_SH_WEDGE | GIC_BASE_ADDR_PB)
    mfc0    k1, C0_EBASE                // Get cp0 EBase
    ext     k1, k1, 0, 10				// Extract CPUNum
    addiu   k1, 0x20					// Offset to base of IPI interrupts.
    sw      k1, 0(k0)					// Clear this IPI.

    la      k0, start_test
    mfc0    k1, C0_EBASE				// Get cp0 EBase
    ext     k1, k1, 0, 10				// Extract CPUNum
    sll     k1, k1, 2
    addu    k0, k0, k1					// index into CMP global "C" variable start_test
    li      k1, 1
    sw      k1, 0(k0)					// Release "cpu"/vpe to execute "C" test code.
   	eret

other_code:
#ifdef RTEST
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait
#else

	// unexpected exception
    mfc0    k1, C0_CAUSE
    mfc0	k0, C0_EPC

    sltiu	zero, zero, 0xabc1	// if using simulator stop with simulator fail condition
    // lui		a5, %hi(0xbfc00000) // Logic Analyzer triger
    // sw		zero, 0(a5)

	sdbbp 					// if probe attached break to debugger
#endif

.org 0x480 # EJTAG debug exception (EJTAG Control Register[ProbTrap] == 0.)
    la 		r24_boston_byte, boston_lcd_display_byte // If post boot then $24 is clobbered.
	// Write to Boston display indicating a exception
 	addu a0,  zero, r23_cpu_num
	li a1, 0x58					// X - Show  exception
	jalr r24_boston_byte

    sltiu	zero, zero, 0xabc1	// if using simulator stop with simulator fail condition
1:  b       1b  				// if no probe attached just spin

.org 0x500 # Resume code past the boot exception vectors.

LEAF(__cpu_init)

#*************************************************************************************
#*************************************************************************************

check_nmi: # Verify we are here due to a reset (and not NMI.)
    mfc0    a0, C0_STATUS				# Read CP0 Status
    srl     a0, 19						# Shift [NMI] into LSBs.
    andi    a0, a0, 1					# Inspect CP0 Status[NMI]
    beqz    a0, verify_isa				# Branch if this is NOT an NMI exception.
    sdbbp								# Failed assertion: not NMI.
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

verify_isa: # Verify device ISA meets code requirements (MIPS32R2 or later.)

#if	defined(BOSTON_RAM)  // make sure ram is up before checking array
	la 	a2, init_FPGA_mem		// Initialize the ROC-it2 MC (Memory Controller.)
	jalr 	a2
#endif

    mfc0    a0, C0_CONFIG				# Read CP0 Config
    srl     a0, 10						# Shift [AT AR] into LSBs.
    andi    a3, a0, 0x18				# Inspect CP0 Config[AT]
    beqz    a3, is_mips32				# Branch if executing on MIPS32R2 ISA.
    sdbbp 								# Failed assertion: MIPS32R2.
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

is_mips32:
    andi    a3, a0, 0x07				# Inspect CP0 Config[AR]
    bnez    a3, init_common_resources	# Continue if ISA is MIPS32R2 or later.
    sdbbp								# Failed assertion MIPS32R2.
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

#*************************************************************************************
# What is initialized on execution depends on the core executing it.
# *************************************************************************************
init_common_resources:  # initializes resources for virtual or physical "cpu".
    la 		a2,init_gpr					# Fill register file with dummy value then boot info.
	jalr 	a2

    la 		a2,	set_gpr_boot_values  # Fill register file boot info.
	jalr 	a2

    la 		a2, init_cp0 				# Init CP0 Status, Count, Compare, Watch*, and Cause.
    jalr 	a2

    la 		a2, init_tlb    			# Generate unique EntryHi contents per entry pair.
    jalr 	a2

    la 		a2, init_gic    			# Configure the global interrupt controller.
    jalr 	a2

	// The L2 cache needs to be disabled because it has not bee initialized
	// Once the Icache has been initialized below caching will be turned on for KSEG0
	// This makes initializing the Dcache and the Icache much faster since the
	// code will be cached. It would also cause the L2 cache to be used before it has
	// been initialized if it were not disabled here.
 	bnez       r8_core_num, init_L1_icache	//  Only done from core 0.
    la 		a2, disable_L2   			# Disable L2 caches
    jalr 	a2

init_L1_icache:
    la 		a2, init_icache 			# Initialize the L1 instruction cache. (Executing using I$ on return.)
    jalr 	a2

# The changing of Kernel mode cacheability must be done from KSEG1
# Since the code is executing from KSEG0 It needs to do a jump to KSEG1 change K0 and jump back to KSEG0
	la 		a2,	change_k0_cca
	li     	a1, 0xf
	ins    	a2, a1, 29, 1 				# changed to KSEG1 address by setting bit 29
    jalr 	a2

    la 		a2, init_dcache 			# Initialize the L1 data cache
    jalr 	a2

    la 		a2, init_itc    			# Initialize Inter-Thread Communications unit
    jalr 	a2

    bnez    r8_core_num, init_sys_resources_done # Only core0/vpe0 needs to init systems resources.

    # core0 only
    la 		a2, init_cpc    			# Initialize the CPS CPC (Cluster Power Controller.)
    jalr 	a2
    la 		a2, init_cm     			# Initialize the CPS CM ( Manager.)
    jalr 	a2

#ifndef RTEST
    la 		a2, copy_c2_ram 			# Copy "C" code and data to RAM and zero bss (uncached.)
    jalr 	a2
#endif

	bnez       r8_core_num, release_cores	//  L2 Only done from core 0.

    la 		a2, init_L2   				# Initialize the unified L2 cache
    jalr 	a2

    la 		a2, enable_L2   			# enable L2 caches
    jalr 	a2

release_cores:
    la 		a2, release_mp  			# Release other cores to execute this boot code.
    jalr 	a2

init_sys_resources_done:				# All Cores
    la 		a2, join_domain 			# Join the Coherence domain.
    jalr 	a2

init_done:
    # Prepare for eret to main (sp and gp set up per vpe in set_gpr_boot_values).
    la      ra, all_done				# If main returns then go to all_done:.
    la      a1, main
    mtc0    a1, C0_ERRPC 				# ErrorEPC

#ifndef RTEST
.set	noat
    # initialize global variable num_cores.
    la      a1, num_cores
    li		a2, 0xFFFFFFFF
    ins     a1, a2, 29, 1 		# Uncached kseg1
    add     a0, r19_more_cores, 1
    sw      a0, 0(a1)
#endif

    # Prepare arguments for main()
    move    a0, r23_cpu_num      		# main(arg0) is the "cpu" number (cp0 EBase[CPUNUM].)

    eret    # Exit reset exception handler for this CPU and start execution of main().

#*************************************************************************************
#*************************************************************************************
all_done:
    # Looks like main returned. Just busy wait spin.
    b       all_done

END(__cpu_init)
