/*
 * copy_c2_SPram.S
 *
 *  Copy code and data to ram then clear BSS
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <mips/m32c0.h>
#include <mips/regdef.h>
#include <mips/cpu.h>
#include <core_config.h>

#define s0_save_C0_ERRCTL 	s0 /* use s0 only to save C0_ERRCTL */
#define all_ones_t4     	t4   /* at Will hold 0xffffffff to simplify bit insertion of 1's. */
#define data_a0    			a0   /* a0 data to be moved */
#define source_addr_a1    	a1   /* from address */
#define destination_addr_a2 a2   /* to address */
#define end_addr_a3    		a3   /* ending address */
#define TEMP_t5				t5
#define CODE_INDEX			s1   /* code index */

	.set	noat                # Don't allow the assembler to use r1(at) for synthetic instr.

/**************************************************************************************
**************************************************************************************/
LEAF(copy_c2_ram)

	li      all_ones_t4, 0xffffffff

    # Copy code from FLASH to ISPram.
    # This is done by "filling" the ISPram
    # simular to "filling" a cache using the cache instruction

    # Position ISPram to code starting address
    # _ftext_ram is the starting address setup in the linker file for the start of the C code
    # NOTE: It is assumed that the address is on a 4K boundary!
	# _ISPram is defined in the linker file and is the physical position for ISPram
	# load the address of the ISPram
	LA		source_addr_a1, _ISPram
    mtc0	source_addr_a1,C0_TAGLO
    # Set err control register SPR bit so cache instructions will
    # be directed to the Scratch Pad registers
    mfc0	s0_save_C0_ERRCTL, C0_ERRCTL
    move    TEMP_t5, s0_save_C0_ERRCTL	# make copy so we can restore C0_ERRCTL
    ins     TEMP_t5, all_ones_t4, 28, 1
    mtc0	TEMP_t5,C0_ERRCTL
    # write instruction tag 0 using the cache instruction
    cache	0x8,0(zero)

    # Next check to see if there is a ISPram
    mfc0	TEMP_t5,C0_CONFIG
    ext 	TEMP_t5, TEMP_t5, 24, 1
    blez	TEMP_t5, set_dspram #no ISPram


    # now start the copy
	# underscore (_) variables come from the linker file
	# zap1 is the starting code address in ROM
	# _ftext_ram is the first instruction address in ram where the code should be copied to
	# _etext_ram is the end instruction address of the code section in ram
    LA      source_addr_a1, _zap1 # starting ROM address
    ins     source_addr_a1, all_ones_t4, 29, 1 # convert to uncached address
    LA      destination_addr_a2, _ftext_ram # starting ram address to copy to
    LA      end_addr_a3, _etext_ram # ending ram address
    beq     destination_addr_a2, end_addr_a3, zero_bss # if both are equal there is nothing to do


    # Need to set the position within the ISPram to write first instructions to
    # _code_index is defined in the linker file
	LA		CODE_INDEX, _code_index

	# get endian mode bit 15 of the config0 register already in $8
    ext 	TEMP_t5, TEMP_t5, 15, 1
    blez	TEMP_t5, next_Iram_wordLE


next_Iram_wordBE:
    lw      data_a0, 0(source_addr_a1)
    mtc0 	data_a0,C0_DATAHI
    addiu   source_addr_a1, 4
    lw      data_a0, 0(source_addr_a1)
    mtc0 	data_a0, C0_DATALO
    cache	0xc,0(CODE_INDEX)
    addiu	CODE_INDEX, 8
    addiu   destination_addr_a2, 8
	addiu   source_addr_a1, 4
    bne		destination_addr_a2, end_addr_a3, next_Iram_wordBE
    b		set_dspram


next_Iram_wordLE:
    lw      data_a0, 0(source_addr_a1)
    mtc0 	data_a0,C0_DATALO
    addiu   source_addr_a1, 4
    lw      data_a0, 0(source_addr_a1)
    mtc0 	data_a0, C0_DATAHI
    cache	0xc,0(CODE_INDEX)
    addiu	CODE_INDEX, 8
    addiu   destination_addr_a2, 8
    addiu   source_addr_a1, 4
    bne		destination_addr_a2, end_addr_a3, next_Iram_wordLE


set_dspram:
    mfc0	TEMP_t5,C0_CONFIG
    ext 	TEMP_t5, TEMP_t5, 23, 1
    blez	TEMP_t5, copy_c2_ram_done #no DSPram just exit

	# Set up for the copy to DSPram initialized data
    # Position ISPram to code starting address
    # _ftext_ram is the starting address setup in the linker file for the start of the C code
    # NOTE: It is assumed that the address is on a 4K boundry!
	# _ISPram is defined in the linker file
	# load the address of the ISPram
	LA		source_addr_a1, _DSPram
	# set the enable bit
    ins     source_addr_a1, all_ones_t4, 7, 1
    # move it to the  tag register
    mtc0	source_addr_a1, C0_DTAGLO
    # write data tag 0 using the cache data controller
    cache	0x9,0(zero)

    # now start the copy
	# underscore (_) variables come from the linker file
	# zap2 is the starting initialized data address in ROM
	# _fdata_ram is the first data address in ram where the code should be copied to
	# _edata_ram is the end data address of the code section in ram
    LA      source_addr_a1, _zap2 # starting ROM address
    ins     source_addr_a1, all_ones_t4, 29, 1 # convert to uncached address
    LA      destination_addr_a2, _fdata_ram # starting ram address to copy to
    LA      end_addr_a3, _edata_ram # ending ram address
    beq     destination_addr_a2, end_addr_a3, zero_bss # if both are equal there is nothing to do


    # Need to set the position within the DSPram array to write data to
    # _data_index is defined in the linker file
	LA		CODE_INDEX, _data_index

next_Dram_word:
    lw      data_a0, 0(source_addr_a1)
    mtc0 	data_a0,C0_DDATALO
    cache	0xd,0(CODE_INDEX)
    addiu	CODE_INDEX, 4
    addiu   destination_addr_a2, 4
    addiu   source_addr_a1, 4
    bne		destination_addr_a2, end_addr_a3, next_Dram_word

zero_bss:
    LA      source_addr_a1, _fbss // start address
    LA      end_addr_a3, _end  // end address
    beq     source_addr_a1, end_addr_a3, copy_c2_ram_done


    # assume bss follows the initialized data
    # write a 0 to the DData Lo register
    mtc0 	zero, C0_DDATALO  // set to 0

next_bss_word:
    cache	0xd,0(CODE_INDEX)      // write DDATA LO to DSPRam
    addiu   source_addr_a1, 4
    addiu	CODE_INDEX, 4          // add 4 to DSPRam index
    bne		source_addr_a1, end_addr_a3, next_bss_word


copy_c2_ram_done:

#ifdef SECONDCOPY
	# initialize global variables:
	// rom_start is the starting code address in ROM; = _slstart_rom
    LA      source_addr_a1, rom_start
    ins		source_addr_a1, zero, 28, 4 # clear top bits because we need index only into DSPRAM
	LA		a2, _etext_ram
	LA 		a3, _ftext_ram
	sub		a2, a2, a3
	LA		a3, _zap1
	add		data_a0, a2, a3

    mtc0 	data_a0,C0_DDATALO
    cache	0xd,0(source_addr_a1)

 	// ram_start is the first instruction address in ram where the code should be copied to; = _f2text_ram
    LA      source_addr_a1, ram_start
    ins		source_addr_a1, zero, 28, 4 # clear top bits because we need index only into DSPRAM
    LA		data_a0, _f2text_ram
    mtc0 	data_a0,C0_DDATALO
    cache	0xd,0(source_addr_a1)

    // ram_end is the end instruction address of the code section in ram; = _e2text_ram
    LA      source_addr_a1, ram_end
    ins		source_addr_a1, zero, 28, 4 # clear top bits because we need index only into DSPRAM
    LA		data_a0, _e2text_ram
    mtc0 	data_a0,C0_DDATALO
    cache	0xd,0(source_addr_a1)
#endif

	# Enable ISPRam
	LA		source_addr_a1, _ISPram
	# set the enable bit
    ins     source_addr_a1, all_ones_t4, 7, 1
    # move it to the tag register
    mtc0	source_addr_a1, C0_TAGLO
    ehb
    # write instruction tag lo using the cache instruction
    cache	0x8,0(zero)
    # restore C0_ERRCTL
    mtc0	s0_save_C0_ERRCTL,C0_ERRCTL
    jr      ra

END(copy_c2_ram)

