/*
 * init_L2_CM2.S
 *
 */
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include <mips/asm.h>
#include <mips/regdef.h>
#include <mips/m32c0.h>
#include <core_config.h>
#include <cm2.h>

#define LINE_SIZE			$3
#define BYTES_PER_LOOP 		$2
#define SET_SIZE			$4
#define ASSOC				$5
#define CONFIG_L2 			$6
#define END_ADDR			$7
#define TOTAL_BYTES			$12
#define	CURRENT_ADDR		$13
#define GCR_BASE_ADDR		$16
#define TEMP				$17

#define LINES_PER_ITER 8 // number of cach instructions per loop

/**************************************************************************************
**************************************************************************************/
LEAF(disable_L2)

	bnez    r8_core_num, done_disable_L2	# Only done from core 0.
    // Disable the L2 cache using by writing a 0x1 to
    // the CM3 L2 Configuration register (ofset 0130)[L2_ByPass (bit 20)]

	lw	    CONFIG_L2, GCR_L2_CONFIG(r22_gcr_addr)	// Read L2 Configuration register
    li      TEMP, 0x1								// set to uncached (by pass)
    ins     CONFIG_L2, TEMP, 20, 1  	  			// Insert bits
    sw      CONFIG_L2, GCR_L2_CONFIG(r22_gcr_addr)	// Write L2 Configuration register
	lw	    CONFIG_L2, GCR_L2_CONFIG(r22_gcr_addr)	// Read L2 Configuration register

done_disable_L2:
    jr      ra
END(disable_L2)
/**************************************************************************************
**************************************************************************************/
LEAF(enable_L2)

	bnez    r8_core_num, done_enable_L2	# Only done from core 0.
    // Enable the L2 cache using by writing a 0x0 to
    // the CM3 L2 Configuration register (ofset 0130)[L2_ByPass (bit 20)]

	lw	    CONFIG_L2, GCR_L2_CONFIG(r22_gcr_addr)	// Read L2 Configuration register
    ins     CONFIG_L2, zero, 20, 1  	  			// Insert bits
    sw      CONFIG_L2, GCR_L2_CONFIG(r22_gcr_addr)	// Write L2 Configuration register
	lw	    CONFIG_L2, GCR_L2_CONFIG(r22_gcr_addr)	// Read L2 Configuration register

done_enable_L2:
    jr      ra
END(enable_L2)
/**************************************************************************************
* Initialize the L2
**************************************************************************************/
LEAF(init_L2)

	bnez    r8_core_num, done_L2_cach_init				// Only done from core 0.

    // Disable the L2 cache using by writing a 0x1 to
    // the CM3 L2 Configuration register (ofset 0130)[L2_ByPass (bit 20)]

	lw	    CONFIG_L2, GCR_L2_CONFIG(r22_gcr_addr)	// Read L2 Configuration register
    li      TEMP, 0x1
    ins     CONFIG_L2, TEMP, 20, 1  	  			// set to uncached (by pass)
    sw      CONFIG_L2, GCR_L2_CONFIG(r22_gcr_addr)	// Write L2 Configuration register
	lw	    TEMP, GCR_L2_CONFIG(r22_gcr_addr)		// Read L2 Configuration register to insure the write has completed

	// Isolate L2$ Line Size
	ext 	LINE_SIZE, CONFIG_L2, 8, 4			// extract LINE_SIZE

	// Skip ahead if No L2$
	beq	    LINE_SIZE, zero, done_L2_cach_init

	li	    TEMP, 2
	sllv	LINE_SIZE, TEMP, LINE_SIZE			// decode for true L2$ line size in bytes


	// Isolate L2$ Sets per Way
	ext		SET_SIZE, CONFIG_L2, 12, 4			// extract SET_SIZE_a0
	li	    TEMP, 64
	sllv	SET_SIZE, TEMP, SET_SIZE			// decode for sets per way


	// Isolate L2$ Associativity
	ext		ASSOC, CONFIG_L2, 0, 4				// extract ASSOC_a1
	addiu	ASSOC,ASSOC, 1						// decode for # of ways

	li 	TEMP, (LINES_PER_ITER)

	mul	    SET_SIZE, SET_SIZE, ASSOC				// Total number of sets
	mul		TOTAL_BYTES, SET_SIZE, LINE_SIZE		// Total number of bytes
	mul		BYTES_PER_LOOP, LINE_SIZE, TEMP		// Total bytes per loop

	// Set the starting address at the beginning of kgeg0 (0x80000000) which will corresponds to
	// way 0 index 0 of the cache and position so starting address is in the
	// middle of the first bytes per loop because the code will use + and - offsets
	lui	    CURRENT_ADDR, 0x8000
	srl		TEMP, BYTES_PER_LOOP, 1
	addu	CURRENT_ADDR, TEMP, CURRENT_ADDR

	addu	END_ADDR, CURRENT_ADDR, TOTAL_BYTES		// make ending address
	subu	END_ADDR, END_ADDR, BYTES_PER_LOOP		// -1 bytes per loop interration

	// Clear L2 Tag registers
	sw 		zero, GCR_L2_TAG_ADDR(r22_gcr_addr)		// GCR_L2_TAG_ADDR
	sw 		zero, GCR_L2_TAG_STATE(r22_gcr_addr)	// GCR_L2_TAG_STATE
	sw 		zero, GCR_L2_DATA(r22_gcr_addr)			// GCR_L2_DATA
	sync
		
// due to offset field restrictions code assumes line size will not be more that 128 bytes for a r5 or less core
// or not more than 64 bytes for a r6 and greater core

next_L2_cache_tag:
	// Index Store Tag Cache Op
	// Will invalidate the tag entry, clear the lock bit, and clear the LRF bit
    cache 0xB, (L2LINE_SIZE*-2)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*-1)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*0)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*1)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*-4)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*-3)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*2)(CURRENT_ADDR)
    cache 0xB, (L2LINE_SIZE*3)(CURRENT_ADDR)

	addu	    CURRENT_ADDR, BYTES_PER_LOOP			// Get next starting line address
	bge	    	END_ADDR, CURRENT_ADDR, next_L2_cache_tag 	// Done yet?

done_L2_cach_init:
    jr      ra
END(init_L2)

#undef LINE_SIZE
#undef BYTES_PER_LOOP
#undef SET_SIZE
#undef ASSOC
#undef CONFIG_L2
#undef END_ADDR
#undef TOTAL_BYTES
#undef	CURRENT_ADDR
#undef TEMP
#undef GCR_BASE_ADDR
#undef LINES_PER_ITER
