/*
 * init_ftlb_R6.S initializations for implementations with a FTLB
 *
 *  Common TLB initialization for MIPS cores
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* This code asumes it is being executed on a proAptiv. This is not portable code!
* It assumes there is always a  TLB and the TLB is at least 64 entries
*/
#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <mips/m32c0.h>

// #defines to make code easier to read
#define FTLB_SIZE_v1 		v1
#define CONFIG4_a0			a0
#define TEMP_a1				a1
#define TEMP_a2 			a2
#define TEMP_a3 			a3
#define VTLB_SIZE_t0		t0
#define CURRENT_INDEX_t1	t1

	.set	noat                // Don't allow the assembler to use r1(at) for synthetic instr.
/**************************************************************************************
**************************************************************************************/
LEAF(init_ftlb)

//  Initialize FTLBs.

	// Determine if we have a FTLB or just a VTLB and set size accordingly

	// determine size of VTLB
	mfc0	VTLB_SIZE_t0, C0_CONFIG, 1
	ext		VTLB_SIZE_t0, VTLB_SIZE_t0, 25, 6 		// t0 VTLB size

	// determine if there is a FTLB
	mfc0    FTLB_SIZE_v1, C0_CONFIG					// read C0_Config
	ext	    FTLB_SIZE_v1, FTLB_SIZE_v1, 7, 3		// check MT (MMU Type) field
	li	    TEMP_a1, 0x4							// MMU Type 4 = dual VLTB + FTLB
	bne	    FTLB_SIZE_v1, TEMP_a1, done_init_ftlb 	// if not, no FTLB to initialize

	// determine size of FTLB
	mfc0	CONFIG4_a0, C0_CONFIG, 4				// Read Config 4
	dext	TEMP_a1, CONFIG4_a0, 0, 4				// get encoded FTLB set size (shift amount)
    dli     TEMP_a3, 1
    dsllv   FTLB_SIZE_v1, TEMP_a3, TEMP_a1          // decode FTLB set size
  	daddu	CURRENT_INDEX_t1, VTLB_SIZE_t0, FTLB_SIZE_v1	// FTLB + VTLB = index of last set in way 0
	addiu	CURRENT_INDEX_t1, CURRENT_INDEX_t1, +1	    	// pre-increment the index (see below)

// The loop decrement operation used to be placed into the delay slot of the branch below with the help
// of a ".set noreorder" directive.  But now this code is written to be reordered by the assembler.  The
// problem is the assembler will not move the decrement operation into the branch delay slot because the
// branch is testing the value of the register being decremented.  So the code has been re-written to let
// the assembler reorder the tlbinvf opcode into the delay slot, and the decrement operation is now at
// the beginning of the loop.  This is why the index has to be pre-incremented above.
next_ftlb_entry_pair:
	addiu	CURRENT_INDEX_t1, CURRENT_INDEX_t1, -1	    	// decrement loop index
	mtc0    CURRENT_INDEX_t1, C0_INDEX						// put loop index into C0_Index
	tlbinvf													// invalidate FTLB set at C0_Index
	bne		CURRENT_INDEX_t1, VTLB_SIZE_t0, next_ftlb_entry_pair	// Done yet? This loop goes one step
																	// past the bottom of the FTLB range,
																	// into the VTLB range.  The last
																	// tlbinvf invalidates the entire VTLB.

done_init_ftlb:
    jalr	zero,ra
END(init_ftlb)

#undef FTLB_SIZE_v1
#undef CONFIG4_a0
#undef TEMP_a1
#undef TEMP_a2
#undef VTLB_SIZE_t0
#undef CURRENT_INDEX_t1

	.set	at                // allow the assembler to use r1(at) for synthetic instr.
