/*
 * init_tlb_FTLB.S initialization
 *
 *  Common TLB initialization for MIPS cores
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/* This code asumes it is being executed on a proAptiv. This is not portable code!
* It assumes there is always a  TLB and the TLB is at least 64 entries
*/
#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <mips/m32c0.h>

	.set	noat                // Don't allow the assembler to use r1(at) for synthetic instr.
/**************************************************************************************
**************************************************************************************/
LEAF(init_tlb)

compute_TLB_size:
	// Determine if we have a FTLB or just a VTLB and set size accordingly
	// if CP0 Config MT = 1 there is only a VTLB and it may be greater than 64K
	// if CP0 Config MT = 4 there is also a FTLB and the VTLB is 64K
	mfc0    v1, C0_CONFIG		// read C0_Config
	ext	    v1, v1, 7, 3		// check MT field
	li	    a3, 0x1				// load a 1 to check against
	bne	    v1, a3, ftlb  		// if not only VTLB branch
	li		v1, 64				// set VTLB entries to 64
	mfc0	a3, C0_CONFIG, 4	// Read Config 4
	ext		a3, a3, 0, 7		// get MMUSizeExt
	sll		a3, a3, 7			// shift to upper bits
	add		v1, v1, a3			// add to the 64
	add	    v1, 1				// pre-increment loop counter because it is decremented
								// at the beginning of the loop instead of at the end
	b		start_init_tlb

ftlb:
	li		v1, 576		// set value of 64 + 512 if using FTLB

start_init_tlb:
	mtc0    zero, C0_ENTRYLO0		// write C0_EntryLo0
	mtc0    zero, C0_ENTRYLO1		// write C0_EntryLo1
	mtc0    zero, C0_PAGEMASK		// write C0_PageMask
	mtc0    zero, C0_WIRED			// write C0_Wired
	li	    a0, 0x80000000

next_tlb_entry_pair:
	add	    v1, -1					// decrement loop counter
    ins     a0, r23_cpu_num, 20, 4	// test: add "cpu" number to provide cps unique entries.
	mtc0    v1, C0_INDEX			// write C0_Index
	mtc0	a0, C0_ENTRYHI			// write C0_EntryHi
	ehb
	tlbwi
	add	    a0, (2<<13)		// Add 8K to the address to avoid TLB conflict with previous entry

// The loop decrement operation used to be placed into the delay slot of the branch below with the help
// of a ".set noreorder" directive.  But now this code is written to be reordered by the assembler.  The
// problem is the assembler will not move the decrement operation into the branch delay slot because the
// branch is testing the value of the register being decremented.  So the code has been re-written with
// the decrement at the beginning of the loop.  Therefore the loop counter has to be pre-incremented
// before entering the loop.
	bne	    v1, zero, next_tlb_entry_pair

done_init_tlb:
    jr      ra
END(init_tlb)
