/* Code to copy Linux package from flash to RAM,
 * Linux package is assumed to contain the kernel busy box file system and device tree
 */
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <string.h>

#define LINUX_SIZE 0xD00000			// should be larger than the vmlinux.bin file
#define DEVICE_TREE_SIZE 0x4000		// should be larger than the boston.dtb file

extern void(boston_lcd_display_byte_ab(unsigned int, char));
void boston_uart_init(void);
void printCopyKernel(void);
void printStartLinux(void);

typedef void (*kernel_entry)(unsigned long a0, unsigned long a1);

void boot_linux(unsigned int cpu_num){
    char *mysrc, *mydst;
    char display_char;

     if (cpu_num==0) {
    	boston_uart_init();

    	// copy kernel
    	printCopyKernel();
        mysrc = (char *) 0xFFFFFFFFBC000000;	// Address in Flash
        mydst = (char *) 0xFFFFFFFF80100000;	// Address code is linked for
        memcpy(mydst, mysrc, LINUX_SIZE);

        // copy device tree
        mysrc = (char *) 0xFFFFFFFFBD000000;	// Address in Flash
        mydst = (char *) 0xFFFFFFFF8FFFC000;    // Address in RAM passed as
								// argument to the run_linux function below
        memcpy(mydst, mysrc, DEVICE_TREE_SIZE);

        printStartLinux();
        // Display an L on the lcd display to indicate the calling of Linux
        display_char = (char)(0x4C);   // ASCII char L for LINUX.
        boston_lcd_display_byte_ab(cpu_num, display_char);

        // Set the run_linux function pointer to the address of the Kernel Entry point
        kernel_entry run_linux = (kernel_entry)0xffffffffA0100000;

        // Call run_linux to start the Linux Kernel
        // with -2 per convention and the address of the Device tree in Ram memory
        run_linux(-2, 0xFFFFFFFF8FFFC000);
    }
}
