/*
 * init_cpc_CM2_64.S
 *
 *  initialization of the Cluster Power Controler
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps.h>

	.set	noat                // Don't allow the assembler to use r1(at) for synthetic instr.

/**************************************************************************************
**************************************************************************************/
LEAF(init_cpc)
    beqz    r11_is_cps, done_init_cpc			// Skip if non-CPS.

    lw      a0, GCR_CPC_STATUS(r22_gcr_addr)	// Read GCR_CPC_STATUS
    andi    a0, 1								// CPC_EX is bit 0
    daddu   r30_cpc_addr, zero, zero
    beqz    a0, done_init_cpc					// Skip if CPC is not implemented (CPC_EX not set)

	// Set address of the CPC in the GCR Configuration GCR_CPC_BASE registers
	// Note: for CM2 the registers are only 32 bits wide so to set the possible 40 bit address
	// 2 store words are needed one to the lower 32 bit register and one to the upper 32 bit register
    dli     a0, CPC_P_BASE_ADDR					// Locate CPC (Physical address)
    sw      a0, GCR_CPC_BASE(r22_gcr_addr)		// Write CPC_BASE address to GCR
    dext	a0, a0, GCR_CPC_BASE_UPPER_BITS, GCR_CPC_BASE_UPPER_BITS_S // extract upper bits of address
	dli 	a1, GCR_CONFIG_ADDR + GCR_CPC_BASE_UPPER // Locate upper register address
	sw		a0, 0(a1)    						// store upper bits of the 40 bit address

    dli     r30_cpc_addr, CPC_BASE_ADDR			// Maintain address of CPC register block.

done_init_cpc:
    jalr zero,     ra
END(init_cpc)
