/*
 * init_gic_64.S
 *
 *  Initialization of the Global Interrupt Conrtoller
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps_CM3.h>

/**************************************************************************************
**************************************************************************************/
LEAF(init_gic)

    beqz    r11_is_cps, done_gic					// Skip if non-CPS.

    daddiu	a1, r22_gcr_addr, GCR_GIC_STATUS
    ld      a0, 0(a1)
    dext    a0, a0, GIC_EX, GIC_EX_S				// Isolate  GCR_GIC_STATUS[GIC_EX].
    beqz    a0, done_gic							// If no gic then skip.

    bnez    r23_cpu_num, init_vpe_gic				// Only core0 vp0 inits shared portion.

    daddiu	a1, r22_gcr_addr, GCR_GIC_BASE
    dla     a0, GIC_P_BASE_ADDR | 1 				// Physical address + enable bit
    sd      a0, 0(a1)

    // Verify gic is 5 "slices" of 8 interrupts giving 40 interrupts.
    dla     a1, GIC_BASE_ADDR						// load GIC KSEG0 must be in uncached mode!
    ld      a0, GIC_SH_CONFIG(a1)					// GIC_SH_CONFIG
    dext    a0, NUMINTERRUPTS, NUMINTERRUPTS_S		// NUMINTERRUPTS (actually slices - 1)
    dli     a3, 4
    bge     a0, a3, configure_slices
    sdbbp   // Failed assertion that gic implements 40 external interrupts.
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

configure_slices:
    // Hardcoded to set up the 24 interrupts  (24..47) for IPI.
    dli     a0, 0x000000ffff000000
    sd      a0, GIC_SH_RMASK63_0(a1)	// (disable     24..47)
    sd      a0, GIC_SH_POL63_0(a1)		// (high/rise   24..47)
    sd      a0, GIC_SH_TRIG63_0(a1)	// (edge        24..47)
    sd      a0, GIC_SH_SMASK63_00(a1)	// (enable      24..47)

    // Initialize configuration of shared interrupts

    // Direct GIC_int 24..47 to processor 0..23
    //  24 interrupts implemented be set aside for IPI signaling from each possible 24 processor.
    // (The actual interrupts are tied low and software sends interrupts via GIC_SH_WEDGE writes.)
    dli	a0, 1             // set bit 0 for processor 0
    sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 24) (a1) // Source 24 to processor  0
    sll	a0, a0, 1          // set bit 1 for processor 1
    sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 25) (a1) // Source 25 to processor  1
    sll	a0, a0, 1          // set bit 2 for processor 2
    sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 26) (a1) // Source 26 to processor  2
    sll	a0, a0, 1          // set bit 3 for processor 3
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 27) (a1) // Source 27 to processor  3

    sll	a0, a0, 1          // set bit 4 for processor 4
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 28) (a1) // Source 28 to processor  4
    sll	a0, a0, 1          // set bit 5 for processor 5
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 29) (a1) // Source 29 to processor  5
    sll	a0, a0, 1          // set bit 6 for processor 6
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 30) (a1) // Source 30 to processor  6
    sll	a0, a0, 1          // set bit 7 for processor 7
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 31) (a1) // Source 31 to processor  7

 	sll	a0, a0, 1          // set bit 1 for processor 8
    sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 32) (a1) // Source 32 to processor  8
    sll	a0, a0, 1          // set bit 2 for processor 9
    sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 33) (a1) // Source 33 to processor  9
    sll	a0, a0, 1          // set bit 3 for processor 10
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 34) (a1) // Source 34 to processor  10
    sll	a0, a0, 1          // set bit 4 for processor 11
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 35) (a1) // Source 35 to processor  11

    sll	a0, a0, 1          // set bit 5 for processor 12
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 36) (a1) // Source 36 to processor  12
    sll	a0, a0, 1          // set bit 6 for processor 13
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 37) (a1) // Source 37 to processor  13
    sll	a0, a0, 1          // set bit 7 for processor 14
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 38) (a1) // Source 38 to processor  14
	sll	a0, a0, 1          // set bit 7 for processor 15
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 39) (a1) // Source 39 to processor  15

 	sll	a0, a0, 1          // set bit 1 for processor 16
    sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 40) (a1) // Source 40 to processor  16
    sll	a0, a0, 1          // set bit 2 for processor 17
    sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 41) (a1) // Source 41 to processor  17
    sll	a0, a0, 1          // set bit 3 for processor 18
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 42) (a1) // Source 42 to processor  18
    sll	a0, a0, 1          // set bit 4 for processor 19
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 43) (a1) // Source 43 to processor  19

    sll	a0, a0, 1          // set bit 5 for processor 20
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 44) (a1) // Source 44 to processor  20
    sll	a0, a0, 1          // set bit 6 for processor 21
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 45) (a1) // Source 45 to processor  21
    sll	a0, a0, 1          // set bit 7 for processor 22
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 46) (a1) // Source 46 to processor  22
	sll	a0, a0, 1          // set bit 7 for processor 23
 	sd	a0, GIC_SH_MAP0_VP63_0 + (GIC_SH_MAP_SPACER * 47) (a1) // Source 47 to processor  23


init_vpe_gic:

    // Initialize configuration of per processor interrupts
    dla     a1, (GIC_BASE_ADDR | GIC_CORE_LOCAL_SECTION_OFFSET)
    lw      a3, GIC_COREL_CTL(a1)

map_timer_int:
    ext     a0, a3, TIMER_ROUTABLE, TIMER_ROUTABLE_S
    beqz    a0, map_perfcount_int

    li      a0, 0x80000005               // Int5 is selected for timer routing
    sw      a0, GIC_COREL_TIMER_MAP(a1)

map_perfcount_int:
    ext     a0, a3, PERFCOUNT_ROUTABLE, PERFCOUNT_ROUTABLE_S
    beqz    a0, done_gic
    li      a0, 0x80000004               // Int4 is selected for performance routing
    sw      a0, GIC_COREL_PERFCTR_MAP(a1)

done_gic:
    jalr	zero,      ra

END(init_gic)
