/*
 * init_gic_CM2_64.S
 *
 *  Initialization of the Global Interrupt Conrtoller
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps.h>

/**************************************************************************************
**************************************************************************************/
LEAF(init_gic)

    beqz    r11_is_cps, done_gic					// Skip if non-CPS.

    dli     a1, GCR_GIC_STATUS + GCR_CONFIG_ADDR	// address of GCR_GIC_STATUS
    lw      a0, 0(a1)
    ext     a0, a0, GIC_EX, GIC_EX_S				// Isolate  GCR_GIC_STATUS[GIC_EX].
    beqz    a0, done_gic							// If no gic then skip.

    bnez    r23_cpu_num, init_vpe_gic				// Only core0 vpe0 inits shared portion.

	// Set address of the GIC in the GCR Configuration GCR_GIC_BASE registers
	// Note: for CM2 the registers are only 32 bits wide so to set the possible 40 bit address
	// 2 store words are needed one to the lower 32 bit register and one to the upper 32 bit register
    dli     a1, GCR_CONFIG_ADDR + GCR_GIC_BASE  	// Locate and enable GIC
    dli     a0, GIC_P_BASE_ADDR | 1 				// Physical address + enable bit
    sw      a0, 0(a1)
    dext	a0, a0, GCR_GIC_BASE_UPPER_BITS, GCR_GIC_BASE_UPPER_BITS_S
    dli 	a1, GCR_CONFIG_ADDR + GCR_GIC_BASE_UPPER  	// Locate upper register address
    sw      a0, 0(a1)

    // Verify gic is 5 "slices" of 8 interrupts giving 40 interrupts.
    dli     a1, GIC_BASE_ADDR						// load GIC KSEG0 must be in uncached mode!
    lw      a0, GIC_SH_CONFIG(a1)					// GIC_SH_CONFIG
    ext     a0, NUMINTERRUPTS, NUMINTERRUPTS_S		// NUMINTERRUPTS (actually slices - 1)
    li      a3, 4
    bge     a0, a3, configure_slices
    sdbbp   // Failed assertion that gic implements 40 external interrupts.
    sltiu $0, $0, 0xabc1	# simulator fail condition
	wait

configure_slices:
    // Hardcoded to set up the last 16 of 40 external interrupts (24..39) for IPI.
    // First interrupts 24... 31 (top 8 bits of 31_0 registers)
    li      a0, 0xff000000
    sw      a0, GIC_SH_RMASK31_0(a1)	// (disable      0..31)
    sw      a0, GIC_SH_POL31_0(a1)		// (high/rise   24..31)
    sw      a0, GIC_SH_TRIG31_0(a1)		// (edge        24..31)
    sw      a0, GIC_SH_SMASK31_00(a1)	// (enable      24..31)

    // Then interrupts 32...40  (lower 8 bits of 63_32 registers)
    li      a0, 0xff
    sw      a0, GIC_SH_RMASK63_32(a1)	// (disable     32..63)
    sw      a0, GIC_SH_POL63_32(a1)		// (high/rise   32..39)
    sw      a0, GIC_SH_TRIG63_32(a1)	// (edge        32..39)
    sw      a0, GIC_SH_SMASK63_32(a1)	// (enable      32..39)

    // Initialize configuration of shared interrupts

    // Direct GIC_int24..39 to vpe 0..7
    // MIPS Linux convention that last 16 interrupts implemented be set aside for IPI signaling.
    // (The actual interrupts are tied low and software sends interrupts via GIC_SH_WEDGE writes.)
    li      a0, 1                        // set bit 0 for CORE0 or for MT vpe0
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 24) (a1) // Source 24 to VPE 0
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 32) (a1) // Source 32 to VPE 0
    sll     a0, a0, 1          // set bit 1 for CORE1 or for MT vpe1
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 25) (a1) // Source 25 to VPE 1
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 33) (a1) // Source 33 to VPE 1
    sll     a0, a0, 1          // set bit 2 for CORE2 or for MT vpe2
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 26) (a1) // Source 26 to VPE 2
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 34) (a1) // Source 34 to VPE 2
    sll     a0, a0, 1          // set bit 3 for CORE3 or for MT vpe3
 	sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 27) (a1) // Source 27 to VPE 3
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 35) (a1) // Source 35 to VPE 3
    sll     a0, a0, 1          // set bit 4 for CORE4 or for MT vpe4
 	sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 28) (a1) // Source 28 to VPE 4
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 36) (a1) // Source 36 to VPE 4
    sll     a0, a0, 1          // set bit 5 for CORE5 or for MT vpe5
 	sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 29) (a1) // Source 29 to VPE 5
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 37) (a1) // Source 37 to VPE 5
    sll     a0, a0, 1          // set bit 6 for CORE6 or for MT vpe6
 	sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 30) (a1) // Source 30 to VPE 6
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 38) (a1) // Source 38 to VPE 6
    sll     a0, a0, 1          // set bit 7 for CORE7 or for MT vpe7
 	sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 31) (a1) // Source 31 to VPE 7
    sw      a0, GIC_SH_MAP0_VPE31_0 + (GIC_SH_MAP_SPACER * 39) (a1) // Source 39 to VPE 7

init_vpe_gic:

    // Initialize configuration of per Core or for MT vpe interrupts
    dli     a1, (GIC_BASE_ADDR | GIC_CORE_LOCAL_SECTION_OFFSET)
    lw      a3, GIC_COREL_CTL(a1)

map_timer_int:
    ext     a0, a3, TIMER_ROUTABLE, TIMER_ROUTABLE_S
    beqz    a0, map_perfcount_int

    li      a0, 0x80000005               // Int5 is selected for timer routing
    sw      a0, GIC_COREL_TIMER_MAP(a1)

map_perfcount_int:
    ext     a0, a3, PERFCOUNT_ROUTABLE, PERFCOUNT_ROUTABLE_S
    beqz    a0, done_gic
    li      a0, 0x80000004               // Int4 is selected for performance routing
    sw      a0, GIC_COREL_PERFCTR_MAP(a1)

done_gic:
    jr      ra

END(init_gic)
