/*
 * join_domain_CM3.S
 *
 *  For CPS cores join processing domain
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps_CM3.h>

	.set	noat                // Don't allow the assembler to use r1(at) for synthetic instr.

/**************************************************************************************
**************************************************************************************/
LEAF(join_domain)

    beqz    r11_is_cps, done_join_domain    // If this is not a CPS then we are done.

	// Enable coherence for this core
	// (Write access enabled via GCR_ACCESS by core 0.)
	li	    a0, 0x01					   // Set Coherent domain enable for 4 cores
	sd	    a0, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_COHERENCE)(r22_gcr_addr)	// GCR_CL_COHERENCE
	ehb

	// Cores other than core 0 can relinquish write access to CM regs here.

#define CORES_LEFT a3
#define VP_LEFT a4

	move CORES_LEFT, r19_more_cores
	move VP_LEFT, r20_more_vpes

	b		set_core_other

next_core:
	addiu 	CORES_LEFT, CORES_LEFT, -1
	move	VP_LEFT, r20_more_vpes
	blt 	CORES_LEFT, zero, done_join_domain

set_core_other:

    // Setup vp and core numbers for other core starting with higest numbers.
    move 	a0, zero
    ins     a0, CORES_LEFT, CORE_OTHER, CORE_OTHER_S
    ins     a0, VP_LEFT, VP_OTHER_NUM, VP_OTHER_NUM_S
    sw      a0, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_OTHER)(r22_gcr_addr) // GCR_CL_OTHER[CoreNum]


busy_wait_coherent_core:

    lw      a0, (CORE_OTHER_CONTROL_BLOCK | GCR_CO_COHERENCE)(r22_gcr_addr) // GCR_CO_COHERENCE
    beqz    a0, busy_wait_coherent_core   // Busy wait on cores joining.

	addiu VP_LEFT, VP_LEFT, -1

	// if last VP on core then on to next core
	blt		VP_LEFT, zero, next_core


done_join_domain:
	jalr	zero,      ra
END(join_domain)

#undef CORES_LEFT
#undef VP_LEFT
