/*
 * join_domain_CM3_64.S
 *
 *  For CPS cores join processing domain
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps_CM3.h>

	.set	noat                // Don't allow the assembler to use r1(at) for synthetic instr.

/**************************************************************************************

Wait for each physical core to enable coherence should only be called by VP0 of each core
**************************************************************************************/
LEAF(join_domain)

    beqz    r11_is_cps, done_enable_coherence    // If this is not a CPS then we are done.

	// Enable coherence for this core
	// (Note: Write access was enabled via GCR_ACCESS by core 0.)
	dli	    a0, 0x01					   	// Set Coherent enable for core
	sd	    a0, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_COH_EN)(r22_gcr_addr)	// GCR_CL_COHERENCE

	daddu  a3, zero, r19_more_cores			// set core number to highest core
	beq	   a3, zero, done_enable_coherence  // branch out if only core 0

next_coherent_core:
    dsll    a0, a3, CORE_OTHER
    sd      a0, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_OTHER)(r22_gcr_addr) // GCR_CL_OTHER[CoreNum]

busy_wait_coherent_core: // Wait for each core in turn to join the domain
    ld      a0, (CORE_OTHER_CONTROL_BLOCK | GCR_CO_COH_EN)(r22_gcr_addr) // GCR_CO_COHERENCE
    beqz    a0, busy_wait_coherent_core   // Busy wait on cores joining.

    daddiu  a3, -1		// decrement core number
    bne     a3, zero, next_coherent_core

done_enable_coherence:

	jalr	zero,      ra
END(join_domain)
