/*
 * power_up_cores_CM3_64.S
 *
 *  Power up other cores
*/
/*
Copyright (c) 2007-2019, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps_CM3.h>
#include <cm3.h>

/**************************************************************************************
**************************************************************************************/
LEAF(power_up_cores)
    dli     a3, 1							// a3 will be the core number to be powered up
    blez    r19_more_cores, done_power_up_next_core   // If no more cores then we are done.
    dli		t0, 1							// Run bit setting

powerup_next_core:
    // set the other core register to the core to be powered up
    daddu   a0, zero, a3			// copy a3 to a0
    dsll    a0, CORE_REDIRECT_SHIFT	// position core number into CORE_REDIRECT field
    sd      a0, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_REDIRECT)(r22_gcr_addr)
    sync

    // power up the other core
    dli     a0, PWR_UP				// "PwrUp" power domain command.
    sd      a0, (CPS_CORE_OTHER_CONTROL_BLOCK | CPC_CMDO_REG)(r30_cpc_addr)
    sync

	// Set go. This is done because core may already be powered up as is the case for a FPGA

	dla     k0, go_vp
	li		k1, 0xA0000000				// make uncached
	or		k0, k0, k1
	li		a0, 16
    dmul    a0, a0, a3					// copy CPUNum and convert it to correct word index
    daddu   k0, a0, k0					// index into the go_vp array
    li		k1, 1						// load 1
    sw		k1, 0(k0)					// set core element of go_vp array
    sync

    // set the Run bit for VP0 of the other core (t0 = 1, setting bit 0 for VP0)
   	sd		t0, (CPS_CORE_OTHER_CONTROL_BLOCK | CPC_CO_VP_RUN_REG)(r30_cpc_addr)
   	sync

    daddiu  a3, a3, 1
    ble     a3, r19_more_cores, powerup_next_core

done_power_up_next_core:
	// See if there is another cluster that needs to be powered up

powerup_next_cluster:
	ld a0, GCR_CONFIG(r22_gcr_addr)
	dext a1, a0, NUM_CLUSTERS_SHIFT, NUM_CLUSTERS_BITS
	dli	a2, 1
	ble a1, a2, done_powerup_next_cluster

done_powerup_next_cluster:
    jalr	zero, ra
END(power_up_cores)
