/*
 * release_mp.S
 *
 *  Release other processors so they can boot
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps.h>

/**************************************************************************************
**************************************************************************************/
LEAF(release_mp)

    blez    r19_more_cores, done_release_mp     // If no more cores then we are done.

    li      a3, 1
    beqz    r30_cpc_addr, release_next_core     // If no CPC then use GCR_CO_RESET_RELEASE

powerup_next_core:
    // Send CPC_RESET command to next core(s) causing execution at their reset exception vector.
    //
    // This code originally used the PwrUp command.  The PwrUp command is OK if there is no
    // debug probe connnected.  But with a probe connected, the power state goes to D2 rather
    // than U2.  When the PwrUp command is issued, the D2 state has a direct path (transition 23)
    // to the U5 power up state without going to the U3 reset state first.  In this case, if
    // core 0 has changed the boot address of other cores using the Reset Exception Base Register,
    // the new boot address won't be used because the Reset Exception Base Register only latches
    // at core reset.  Using the Reset command ensures that the core goes through the reset state.
    move    a0, a3
    sll     a0, 16
    sw      a0, (CPS_CORE_LOCAL_CONTROL_BLOCK | CPC_OTHERL_REG)(r30_cpc_addr)
    li      a0, CPC_RESET		// "CPC_RESET" Reset command to force to U3 reset then power up.
    sw      a0, (CPS_CORE_OTHER_CONTROL_BLOCK | CPC_CMDO_REG)(r30_cpc_addr)
    addiu   a3, a3, 1
    bge     r19_more_cores, a3, powerup_next_core

    jalr	zero, ra

release_next_core:
    // Release next core to execute at their reset exception vector.
    move    a0, a3
    sll     a0, 16
    sw      a0, (CORE_LOCAL_CONTROL_BLOCK | GCR_CL_OTHER)(r22_gcr_addr) // GCR_CL_OTHER
    sw      zero, 0x4000(r22_gcr_addr)           // GCR_CO_RESET_RELEASE
    addiu   a3, a3, 1
    bge     a3, r19_more_cores, release_next_core

done_release_mp:
    jalr	zero, ra
END(release_mp)
