/*
 * start_VPs_CM3_64.S
 *
 *  Release other processors so they can boot
*/
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <mips/asm.h>
#include <core_config.h>
#include <mips/regdef.h>
#include <cps_CM3.h>

	.set	noat                // Don't allow the assembler to use r1(at) for synthetic instr.
/**************************************************************************************
**************************************************************************************/
LEAF(start_vps)

	// set VP_RUN bits
	dli     a3, 1
    // start remaining VPs of the core
	beqz r20_more_vpes, done_start_vps  // if no other VPs - done with this section

	// create bit mask from number of VP on core
	daddu	a0, a3, r20_more_vpes	// add 1 to include VP0
	dsllv	a0, a3, a0				// shift 1 by number of VPs
	daddiu	a0, a0, -1				// subtract 1 to complete mask

    sd      a0, (CPS_CORE_LOCAL_CONTROL_BLOCK | CPC_CL_VP_RUN_REG)(r30_cpc_addr)

	daddu	a0, zero, r20_more_vpes	//	copy number of vp on this core

next_vp:
	// Set go. This is done because core may already be powered up as is the case for a FPGA
	dla     k0, go_vp
	li		k1, 0xA0000000			// make uncached
	or		k0, k0, k1
    daddu   k1, a0, r23_cpu_num		// CPUNum + vp(a0) = VP in system
    dsll    k1, k1, 2				// x 4 for integer element
    daddu   k0, k0, k1				// index into the go_vp array
    li		k1, 1					// load 1
    sw		k1, 0(k0)				// set core element of go_vp array
    sync
    addiu	a0, a0, -1				// subtract 1 from the VP on this core
    bnezc	a0, next_vp				// loop again if not equal to 0

done_start_vps:
    jalr	zero, ra
END(start_vps)
