
/*
Copyright (c) 2007-2018, MIPS Tech, LLC and/or its affiliated group companies or licensors

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other materials
provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors may be
used to endorse or promote products derived from this software without specific prior
written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// Terminology
// VPE:        (virtual processing element): Core functionality necessary to support privileged execution (OS.)
// TC:         (thread context): Core functionality necessary to support a thread of execution.
// CPU:        A core vpe/tc pair.

// Defines

// Global variables.
// volatile unsigned int rom_start, ram_start, ram_end;
volatile int ready[8] ;
volatile int boot_count[8] = { 1, 2, 3, 4, 5, 6, 7, 8};

void boston_lcd_display_byte_ab(unsigned int, char);

//
// main(): Synchronized run of shared test code coordinated by cpu0.
//
int main(unsigned int vpe_num, unsigned int num_vpe_this_core) {
    
    int i, j, k ;
    int num_cpus = 0 ;
    char display_char, blank;
    int temp;

    // End timing of boot for this "cpu".
    asm volatile ("mfc0 %[temp], $9": [temp] "=r"(temp) :) ;
    boot_count[vpe_num] = temp ;

    // VPE0 synchronizes test code execution.
    if (vpe_num == 0) {

        // Wait for other VPEs to indicate they are ready.
        for (i = 1; i < num_vpe_this_core; i++) {
        	boston_lcd_display_byte_ab(vpe_num, 'W'); // 'W' for Waiting.
            while (!ready[i]) ;         // Busy wait for all VPEs to be ready.
        }
        
    } else {

        // Other VPEs indicate ready and wait...
    	boston_lcd_display_byte_ab(vpe_num, 'r');     // display 'r' for ready.
        ready[vpe_num] = 1 ;
    }

    // Put test code here:

    while (1) {
        display_char = (char)(vpe_num + 0x30);   // ASCII char for cpu number.
        blank = (char)0x20;
        for (j = 0 ; j < (4 * 1024) ; j++) {
        	boston_lcd_display_byte_ab(vpe_num, display_char);
        }
        for (j = 0 ; j < (4 * 1024) ; j++) {
        	boston_lcd_display_byte_ab(vpe_num, blank); // blank out display position for this "cpu".
        }
    }

    return 0 ;
}
