
#include <mips/asm.h>

#include <mips/regdef.h>
#include <core_config.h>

///////////////////////////////////////////////////////////////////////////////////
//
// initTestRun
// set K0 CCA to 2 (uncached)
//
///////////////////////////////////////////////////////////////////////////////////

LEAF(initTestRun)

initTestRun:
	mfc0 t0, C0_CONFIG
	li t1, 2
	ins t0, t1, 0, 3
	mtc0 t0, C0_CONFIG
	jr ra

END(initTestRun)

///////////////////////////////////////////////////////////////////////////////////
//
// disableInterrupt
//
// non return function take one argument, a0=cache size in byte
// did not do range checking for the a0
//
///////////////////////////////////////////////////////////////////////////////////

LEAF(disableInterrupt)

disableInterrupt:
	di
	ehb
	jr ra

END(disableInterrupt)

///////////////////////////////////////////////////////////////////////////////////
//
// enableInterrupt
//
// non return function take one argument, a0=cache size in byte
// did not do range checking for the a0
//
///////////////////////////////////////////////////////////////////////////////////
LEAF(enableInterrupt)

enableInterrupt:
	ei
	ehb
	jr ra

END(enableInterrupt)

///////////////////////////////////////////////////////////////////////////////////
// Return ICache size
//
///////////////////////////////////////////////////////////////////////////////////

#define LINE_SIZE_v1		v1
#define SET_SIZE_a0			a0
#define ASSOC_a1			a1
#define CONFIG1_a2 			a2
#define TEMP1_a3			a3
#define TOTAL_BYTES_v0		v0  // return value

LEAF(icache_size)

	// Determine how big the I$ is
	mfc0	CONFIG1_a2, C0_CONFIG1						// read C0_Config1

	li    LINE_SIZE_v1, ILINE_SIZE						// line size in bytes (core_config.h)

	ext	    SET_SIZE_a0, CONFIG1_a2, CFG1_ISSHIFT, 3	// extract IS
	li	    TEMP1_a3, 64
	sllv    SET_SIZE_a0, TEMP1_a3, SET_SIZE_a0				// I$ Sets per way

	// Config1IA == I$ Assoc - 1
	ext	    ASSOC_a1, CONFIG1_a2, CFG1_IASHIFT, 3		// extract IA
	addiu	ASSOC_a1, ASSOC_a1, 1

	mul	    SET_SIZE_a0, SET_SIZE_a0, ASSOC_a1			// Total number of sets
	mul		TOTAL_BYTES_v0, SET_SIZE_a0, LINE_SIZE_v1		// Total number of bytes
	jr ra

END(icache_size)

///////////////////////////////////////////////////////////////////////////////////
// Return DCache size
//
///////////////////////////////////////////////////////////////////////////////////
LEAF(dcache_size)

	// Determine how big the I$ is
	mfc0	CONFIG1_a2, C0_CONFIG1						// read C0_Config1

	li    LINE_SIZE_v1, DLINE_SIZE						// line size in bytes (core_config.h)

	ext	    SET_SIZE_a0, CONFIG1_a2, CFG1_DSSHIFT, 3	// extract IS
	li	    TEMP1_a3, 64
	sllv    SET_SIZE_a0, TEMP1_a3, SET_SIZE_a0			// I$ Sets per way

	// Config1IA == I$ Assoc - 1
	ext	    ASSOC_a1, CONFIG1_a2, CFG1_DASHIFT, 3		// extract IA
	addiu	ASSOC_a1, ASSOC_a1, 1

	mul	    SET_SIZE_a0, SET_SIZE_a0, ASSOC_a1			// Total number of sets
	mul		TOTAL_BYTES_v0, SET_SIZE_a0, LINE_SIZE_v1	// Total number of bytes
	jr ra

END(dcache_size)
///////////////////////////////////////////////////////////////////////////////////
//
// invalidateICache
//
// non return function take one argument, a0=cache size in byte
// did not do range checking for the a0
//
///////////////////////////////////////////////////////////////////////////////////

LEAF(invalidateICache)

invalidateICache:
	// prepare loop variable, 32 byte alignment
	li t0, 0x80000000
	add t0, t0, a0
	srl t0, t0, 5
	sll t0, t0, 5
	// zero base, remove last
	addiu t0, t0, -32

	li t1, 0x7fffffe0

1:
	addiu t1, t1, 32
	cache Index_Invalidate_I, 0(t1)
	bne t0, t1, 1b

	jr ra
END(invalidateICache)

///////////////////////////////////////////////////////////////////////////////////
//
// invalidateDCache
//
// non return function take one argument, a0=cache size in byte
// did not do range checking for the a0
//
///////////////////////////////////////////////////////////////////////////////////
LEAF(invalidateDCache)
invalidateDCache:
	// prepare loop variable, 32 byte alignment
	li t0, 0x80000000
	add t0, t0, a0
	srl t0, t0, 5
	sll t0, t0, 5
	// zero base, remove last
	addiu t0, t0, -32

	li t1, 0x7fffffe0

1:
	addiu t1, t1, 32
	cache Index_Writeback_Inv_D, 0(t1)
	bne t0, t1, 1b

	jr ra
END(invalidateDCache)

///////////////////////////////////////////////////////////////////////////////////
//
// invalidateL2Cache
//
// non return function take one argument, a0=cache size in byte
// did not do range checking for the a0
//
///////////////////////////////////////////////////////////////////////////////////
LEAF(invalidateL2Cache)
invalidateL2Cache:
	// prepare loop variable, 32 byte alignment
	li t0, 0x80000000
	add t0, t0, a0
	srl t0, t0, 5
	sll t0, t0, 5
	// zero base, remove last
	addiu t0, t0, -32

	li t1, 0x7fffffe0

1:
	addiu t1, t1, 32
	cache Index_Writeback_Inv_S, 0(t1)
	bne t0,t1, 1b

	jr ra
END(invalidateL2Cache)

///////////////////////////////////////////////////////////////////////////////////
//
// fillLockL2Line32
//
// non return function take two arguments, a0=data pointer, a1=physical address
// fill L2 entry with the data pointed by the data/inst pointer to the physical address, and lock the entry
// the data/inst can be hard code in the program
// L2 line size is 32 bytes
///////////////////////////////////////////////////////////////////////////////////
LEAF(fillLockL2Line32)
fillLockL2Line32:

	// word alignment, cachable VA from PA
	li t0, 0x1FFFFFFC
	and t0, t0, a1
	li t1, 0x80000000
	or t0, t0, t1

	// word align, pointer address VA, assume VA is addressable
	li t1, 0xFFFFFFFC
	and t1, t1, a0

1:
	// set WST, SPR=0
	mfc0 t2, C0_ERRCTL
	li t3, 0xCFFFFFFF
	and t2, t2, t3
	mtc0 t2, C0_ERRCTL
	ehb

	// set valid and lock bit
	li t2, 0xA0
	or t2, t2, a1
	mtc0 t2, C0_TAGLO2
	ehb

	// store tag
	cache Index_Store_Tag_S, 0(t0)
	ehb

	// store data
	lw t2, 0(t1)
	lw t3, 4(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 0(t0)
	ehb

	// store data
	lw t2, 8(t1)
	lw t3, 12(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 8(t0)
	ehb

	// store data
	lw t2, 16(t1)
	lw t3, 20(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 16(t0)
	ehb

	// store data
	lw t2, 24(t1)
	lw t3, 28(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 24(t0)
	ehb

	jr ra
END(fillLockL2Line32)

///////////////////////////////////////////////////////////////////////////////////
//
// fillLockL2Line64
//
// non return function take two arguments, a0=data pointer, a1=physical address
// fill L2 entry with the data pointed by the data/inst pointer to the physical address, and lock the entry
// the data/inst can be hard code in the program
// L2 line size is 64 bytes
///////////////////////////////////////////////////////////////////////////////////
LEAF(fillLockL2Line64)
fillLockL2Line64:

	// word alignment, cachable VA from PA
	li t0, 0x1FFFFFFC
	and t0, t0, a1
	li t1, 0x80000000
	or t0, t0, t1

	// word align, pointer address VA, assume VA is addressable
	li t1, 0xFFFFFFFC
	and t1, t1, a0

1:
	// set WST, SPR=0
	mfc0 t2, C0_ERRCTL
	li t3, 0xCFFFFFFF
	and t2, t2, t3
	mtc0 t2, C0_ERRCTL
	ehb

	// set valid and lock bit
	li t2, 0xA0
	or t2, t2, a1
	mtc0 t2, C0_TAGLO2
	ehb

	// store tag
	cache Index_Store_Tag_S, 0(t0)
	ehb

	// store data
	lw t2, 0(t1)
	lw t3, 4(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 0(t0)
	ehb

	// store data
	lw t2, 8(t1)
	lw t3, 12(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 8(t0)
	ehb

	// store data
	lw t2, 16(t1)
	lw t3, 20(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 16(t0)
	ehb

	// store data
	lw t2, 24(t1)
	lw t3, 28(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 24(t0)
	ehb

	// store data
	lw t2, 32(t1)
	lw t3, 36(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 32(t0)
	ehb

	// store data
	lw t2, 40(t1)
	lw t3, 44(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 40(t0)
	ehb

	// store data
	lw t2, 48(t1)
	lw t3, 52(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 48(t0)
	ehb

	// store data
	lw t2, 56(t1)
	lw t3, 60(t1)
	mtc0 t2, C0_DATALO2
	mtc0 t3, $29, 5
	cache 0xF, 56(t0)
	ehb

	jr	ra
END(fillLockL2Line64)

///////////////////////////////////////////////////////////////////////////////////
//
// disableL2Bypass
//
// non return function take no arguments, disable L2 bypass, assume L2 are available
//
///////////////////////////////////////////////////////////////////////////////////
LEAF(disableL2Bypass)
disableL2Bypass:

	mfc0 t0, C0_CONFIG2
	li t1, 0xFFFFEFFF
	and t0, t0, t1
	mtc0 t0, C0_CONFIG2
	nop
	nop

	add t0, t0, 0x100
	add t0, t0, 0x200
	add t0, t0, 0x300
	add t0, t0, 0x400
	add t0, t0, 0x500
	add t0, t0, 0x600
	add t0, t0, 0x700
	add t0, t0, 0x800

	jr ra
END(disableL2Bypass)


///////////////////////////////////////////////////////////////////////////////////
//
// ICacheMScan
// argument a0 = icache_size
//
// return v0 = 0 if pass, return non zero if fail
// Perform MSCan test on I$ which has the pattern of {write 0, read 0, write 1, read 1}. This routine will
// test tag ram then the data ram
//
///////////////////////////////////////////////////////////////////////////////////
LEAF(ICacheMScan)
ICacheMScan:

ICacheMScanTag:
	li	t1, 2
	mfc0	t0, C0_CONFIG0
	ins t0, t1, 0, 3
	mtc0 t0, C0_CONFIG0
	// prepare to test the tag array, w0
	// The ErrCtl register controls parity protection of data and instruction caches
	// and provides for software testing of the way-selection
	// ErrCtl[PE(31)]=0, Parity or ECC disabled
	// ErrCtl[PO(30)]=1, Override calculated parity
	// ErrCtl[WST(29)]=0, cache IndexLoadTag/cache IndexStoreTag instructions read/write cache tags
	// ErrCtl[PCO(27)]=1, set precode override. Used for diagnostic/test of the instruction cache
	// parity disable, parity overwrite, test mode disable, overwrite precode
	li t0, 0x48000000
	mtc0 t0, C0_ERRCTL

	// The ITagLo register acts as the interface to the instruction cache tag array.
	// The Index Store Tag and Index Load Tag operations of the CACHE instruction use the ITagLo
	// register as the source of tag information
	// Write of zero clears all fields:
	// PTagLo(31-10) physical address of the cache line
	// V (7) valid bit to invalid
	// L (5) Clear lock bit
	// P (0) Parity bit over the cache tag entries
	mtc0 zero, C0_ITAGLO

	// prepare loop variable, 32 byte alignment,
	// t1 is loop start variable
	// t0 is the loop end variable
	li t1, 0x80000000  // set start variable
	add t0, t1, a0	   // add cache size to start
	// align to 32 bytes by clearing bottom 5 bits
	ins	t0, zero, 0, 5
	move t4, t1
1:	// loop sets all instruction tags to 0
	// update tag ram from ErrCtl indexed by t4
	addi t4, t4, 32 // increment to next cache line
	cache Index_Store_Tag_I, -32(t4)
	bne t0,t4, 1b	// test for last cache line

	// start r0 test
	// reset end segment
	move t4, t1
	move t3, zero
1:	// loop reads instruction tags and tests they are 0
	cache Index_Load_Tag_I, 0(t4)
	ehb
	addi t4, t4, 32	// increment to next cache line
	mfc0 t2, C0_ITAGLO
	or t3, t3, t2
	bne t0,t4, 1b	// test for last cache line

	beq zero, t3, 2f // if reads all zero then branch to next test
	
	// end test with return =1 if any read was not 0
	li v0, 1
	jr ra

2:
	// start w1 test
	li t2, 0xFFFFFFFF
	mtc0 t2, C0_ITAGLO
	move t4, t1
1:
	// update tag ram from ErrCtl
	addi t4, t4, 32
	cache Index_Store_Tag_I, -32(t4)
	bne t0,t4, 1b

	// start r1 test
	move t4, t1
	li t3, 0xffffffff
1:
	cache Index_Load_Tag_I, 0(t4)
	ehb
	addi t4, t4, 32
	mfc0 t2, C0_ITAGLO
	and t3, t3, t2
	bne t0, t4, 1b

	// end test with return =2 if doesn't matched
	li t2, 0xFFFFF0A1
	beq t2, t3, ICacheMScanData
	li v0, 2
	jr ra

ICacheMScanData:
	// when reach here, pass tag ram mscan test
	// start data ram mscan test

	// prepare to test the data array, w0
	// ErrCtl[PE]=0,ErrCtl[PO]=1,ErrCtl[WST]=1, ErrCtl[PCO]=1
	// parity disable, parity overwrite, test mode enable, overwrite precode
	li t2, 0x68000000
	mtc0 t2, C0_ERRCTL
	mtc0 zero, C0_IDATALO
	mtc0 zero, C0_IDATAHI
	move t4, t1
1:
	addi t4, t4, 32
	// update data ram from IDataLo and IDataHi
	cache 0xC, -32(t4)
	// update 2nd 64 bit in I$ data ram
	cache 0xC, -24(t4)
	// update 3rd 64 bit in I$ data ram
	cache 0xC, -16(t4)
	// update 4th 64 bit in I$ data ram
	cache 0xC, -8(t4)
	bne t0, t4, 1b

	// start r0 test
	// prepare loop variable, 32 byte alignment, t1 is loop variable
	// result store in t3
	// switch to non test mode
	li t2, 0x48000000
	mtc0 t2, C0_ERRCTL
	move t3, zero
	move t4, zero
	// mask for PCI, PI bits
	li t5, 0x7FFF0
	move t6, t1
1:
	cache Index_Load_Tag_I, 0(t6)
	ehb
	mfc0 t2, C0_IDATALO
	or t3, t3, t2
	mfc0 t2, C0_IDATAHI
	or t3, t3, t2

	// 2nd 64 bit 
	cache Index_Load_Tag_I, 8(t6)
	ehb
	mfc0 t2, C0_IDATALO
	or t3, t3, t2
	mfc0 t2, C0_IDATAHI
	or t3, t3, t2

	// 3rd 64 bit 
	cache Index_Load_Tag_I, 16(t6)
	ehb
	mfc0 t2, C0_IDATALO
	or t3, t3, t2
	mfc0 t2, C0_IDATAHI
	or t3, t3, t2

	// 4th 64 bit 
	cache Index_Load_Tag_I, 24(t6)
	ehb
	mfc0 t2, C0_IDATALO
	or t3, t3, t2
	mfc0 t2, C0_IDATAHI
	or t3, t3, t2

	addi t6, t6, 32

	mfc0 t2, C0_ERRCTL
	and t2, t2, t5
	or t4, t4, t2
	bne t0,t6, 1b

	// end test with return =3 if doesn't matched
	beq zero, t3, 3f
	
	li v0, 3
	jr ra

3:
	// end test with return =4 if PCI/PI doesn't matched
	beq zero, t4, 5f

	li v0, 4
	jr ra
	nop

5:

	// start w1 test
	// prepare to test the tag array, w0
	// ErrCtl[PE]=0,ErrCtl[PO]=1,ErrCtl[WST]=1, ErrCtl[PCO]=1
	// parity disable, parity overwrite, test mode disable, overwrite precode, set PCI and PI all bits 1
	li t2, 0x6807FFF0
	
	mtc0 t2, C0_ERRCTL
	li t2, 0xFFFFFFFF
	mtc0 t2, C0_IDATALO
	mtc0 t2, C0_IDATAHI
	move t4, t1
1:
	addi t4, t4, 32

	// update data ram from ErrCtl
	cache 0xC, -32(t4)
	// update 2nd 64bit I$ data ram
	cache 0xC, -24(t4)
	// update 3rd 64bit I$ data ram
	cache 0xC, -16(t4)
	// update 4th 64bit I$ data ram
	cache 0xC, -8(t4)
	// loop end
	bne t0,t4, 1b

	// start r1 test
	// prepare loop variable, 32 byte alignment, t1 is loop variable
	// result store in t3
	// stay at test mode
	li t2, 0x6807FFF0
	mtc0 t2, C0_ERRCTL

	li t3, 0xFFFFFFFF
	li t4, 0xFFFFFFFF
	
	// mask for PCI, PI bits
	li t5, 0x7FFF0

	move t6, t1
1:
	cache Index_Load_Tag_I, 0(t6)
	ehb
	mfc0 t2, C0_IDATALO
	and t3, t3, t2
	mfc0 t2, C0_IDATAHI
	and t3, t3, t2

	// 2nd 64 bit
	cache Index_Load_Tag_I, 8(t6)
	ehb
	mfc0 t2, C0_IDATALO
	and t3, t3, t2
	mfc0 t2, C0_IDATAHI
	and t3, t3, t2

	// 3rd 64 bit
	cache Index_Load_Tag_I, 16(t6)
	ehb
	mfc0 t2, C0_IDATALO
	and t3, t3, t2
	mfc0 t2, C0_IDATAHI
	and t3, t3, t2

	// 4th 64 bit
	cache Index_Load_Tag_I, 24(t6)
	ehb
	mfc0 t2, C0_IDATALO
	and t3, t3, t2
	mfc0 t2, C0_IDATAHI
	and t3, t3, t2

	addi t6, t6, 32

	mfc0 t2, C0_ERRCTL
	and t2, t2, t5
	and t4, t4, t2
	bne t0,t6, 1b

	// end test with return =5 if doesn't matched
	li t2, 0xFFFFFFFF
	beq t2, t3, 3f
	nop
	
	li v0, 5
	jr ra

3:
	beq t5, t4, ICacheMScanPass

4:
	li v0, 6
	jr ra

ICacheMScanPass:

	// return test with value 0 if no problem
	li v0, 0
	jr ra

END(ICacheMScan)


///////////////////////////////////////////////////////////////////////////////////
//
// ICacheMScan1
//
// return 0 if pass, return non zero if fail
// Perform MSCan test on I$ which has the pattern of {w0, r0, w1, r1}. This routine will 
// test tag ram then the data ram
//
///////////////////////////////////////////////////////////////////////////////////
LEAF(ICacheMScan1)
	.set    noreorder
ICacheMScan1:


ICacheMScanTag1:
	// prepare to test the tag array, w0
	// ErrCtl[PE]=0,ErrCtl[PO]=1,ErrCtl[WST]=0, ErrCtl[PCO]=1
	// parity disable, parity overwrite, test mode disable, overwrite precode
	li t0, 0x48000000
	
	mtc0 t0, C0_ERRCTL
	mtc0 zero, C0_ITAGLO

	// prepare loop variable, 32 byte alignment, t1 is loop variable
	li t0, 0x80000000
	add t0, t0, a0
	srl t0, t0, 5
	sll t0, t0, 5
	// zero base, remove last 
	addiu t0, t0, -32

	li t1, 0x80000000
	move t6, zero
1:
	
	// update tag ram from ErrCtl
	cache Index_Store_Tag_I, 0(t1)

	addiu t6, t6,0x1
	sll t7, t6, 12
	mtc0 t7, C0_ITAGLO

	bne t0,t1, 1b
	addi t1, t1, 32


	// start r0 test
	// prepare loop variable, 32 byte alignment, t1 is loop variable
	// result store in t3
	li t0, 0x80000000
	add t0, t0, a0
	srl t0, t0, 5
	sll t0, t0, 5
	// zero base, remove last 
	addiu t0, t0, -32

	li t1, 0x80000000
	
	move t2, zero
	move t3, zero
	move t6, zero
	move t7, zero
	
1:
	

	cache Index_Load_Tag_I, 0(t1)
	ehb
	mfc0 t2, C0_ITAGLO
	addu t3, t2, t3
	
	addu t6, t6, t7
	addiu t7, t7, 0x1


	bne t0,t1, 1b
	addi t1, t1, 32

	sll t6, t6, 12

	// end test with return =1 if not matched
	beq t3, t6, ICacheMScanData1
	nop
	
2:
	li v0, 1
	jr ra
	nop



ICacheMScanData1:
	// when reach here, pass tag ram mscan test
	// start data ram mscan test

	// prepare to test the data array, w0
	// ErrCtl[PE]=0,ErrCtl[PO]=1,ErrCtl[WST]=1, ErrCtl[PCO]=1
	// parity disable, parity overwrite, test mode enable, overwrite precode
	li t0, 0x68000000
	move t6, zero
	
	mtc0 t0, C0_ERRCTL
	mtc0 t6, C0_IDATALO
	addiu t6, t6, 1
	mtc0 t6, C0_IDATAHI

	// prepare loop variable, 32 byte alignment, t1 is loop variable
	li t0, 0x80000000
	add t0, t0, a0
	srl t0, t0, 5
	sll t0, t0, 5
	// zero base, remove last 
	addiu t0, t0, -32

	li t1, 0x80000000
	
1:
	
	// update data ram from ErrCtl
	cache 0xC, 0(t1)

	// update 2nd 64 bit in I$ data ram
	addiu t6, t6, 1 
	mtc0 t6, C0_IDATALO
	addiu t6, t6, 1 
	mtc0 t6, C0_IDATAHI
	cache 0xC, 8(t1)

	// update 3rd 64 bit in I$ data ram
	addiu t6, t6, 1 
	mtc0 t6, C0_IDATALO
	addiu t6, t6, 1 
	mtc0 t6, C0_IDATAHI
	cache 0xC, 16(t1)

	// update 4th 64 bit in I$ data ram
	addiu t6, t6, 1 
	mtc0 t6, C0_IDATALO
	addiu t6, t6, 1 
	mtc0 t6, C0_IDATAHI
	cache 0xC, 24(t1)

	addiu t6, t6, 1 
	mtc0 t6, C0_IDATALO
	addiu t6, t6, 1 
	mtc0 t6, C0_IDATAHI

	bne t0,t1, 1b
	addiu t1, t1, 32

	// start r0 test
	// prepare loop variable, 32 byte alignment, t1 is loop variable
	// result store in t3

	// switch to non test mode
	li t0, 0x48000000
	mtc0 t0, C0_ERRCTL

	li t0, 0x80000000
	add t0, t0, a0
	srl t0, t0, 5
	sll t0, t0, 5
	// zero base, remove last 
	addiu t0, t0, -32

	li t1, 0x80000000
	
	move t2, zero
	move t3, zero
	move t4, zero
	move t6, zero
	move t7, zero
	
	// mask for PCI, PI bits
	li t5, 0x7FFF0
1:
	cache Index_Load_Tag_I, 0(t1)
	ehb
	mfc0 t2, C0_IDATALO
	add t3, t3, t2
	mfc0 t2, C0_IDATAHI
	add t3, t3, t2
	add t6, t6, t7
	addiu t7, t7, 1
	add t6, t6, t7
	addiu t7, t7, 1

	// 2nd 64 bit 
	cache Index_Load_Tag_I, 8(t1)
	ehb
	mfc0 t2, C0_IDATALO
	add t3, t3, t2
	mfc0 t2, C0_IDATAHI
	add t3, t3, t2
	add t6, t6, t7
	addiu t7, t7, 1
	add t6, t6, t7
	addiu t7, t7, 1

	// 3rd 64 bit 
	cache Index_Load_Tag_I, 16(t1)
	ehb
	mfc0 t2, C0_IDATALO
	add t3, t3, t2
	mfc0 t2, C0_IDATAHI
	add t3, t3, t2
	add t6, t6, t7
	addiu t7, t7, 1
	add t6, t6, t7
	addiu t7, t7, 1

	// 4th 64 bit 
	cache Index_Load_Tag_I, 24(t1)
	ehb
	mfc0 t2, C0_IDATALO
	add t3, t3, t2
	mfc0 t2, C0_IDATAHI
	add t3, t3, t2
	add t6, t6, t7
	addiu t7, t7, 1
	add t6, t6, t7
	addiu t7, t7, 1

	mfc0 t2, C0_ERRCTL
	and t2, t2, t5
	or t4, t4, t2

	bne t0,t1, 1b
	addi t1, t1, 32

	// end test with return =3 if not matched
	beq t6, t3, 3f
	nop
	
2:
	li v0, 3
	jr ra
	nop

3:
	// end test with return =4 if PCI/PI not matched
	beq zero, t4, ICacheMScanPass1
	nop

4:
	li v0, 4
	jr ra
	nop



ICacheMScanPass1:

	// return test with value 0 if no problem
	li v0, 0
	jr ra
	nop

	.set    reorder
END(ICacheMScan1)






///////////////////////////////////////////////////////////////////////////////////
//
// DCacheMScan
//
// return 0 if pass, return non zero if fail
// Perform MSCan test on D$ which has the pattern of {w0, r0, w1, r1}. This routine will 
// test tag ram then the data ram
//
///////////////////////////////////////////////////////////////////////////////////
LEAF(DCacheMScan)
	.set    noreorder
DCacheMScan:


DCacheMScanTag:
	// prepare to test the tag array, w0
	// ErrCtl[PE]=0,ErrCtl[PO]=1,ErrCtl[WST]=0, ErrCtl[PCO]=1
	// parity disable, parity overwrite, test mode disable, overwrite precode
	li t0, 0x48000000
	
	mtc0 t0, C0_ERRCTL
	mtc0 zero, C0_DTAGLO
	// prepare loop variable, 32 byte alignment,
	// t1 is loop start variable
	// t0 is the loop end variable
	li t1, 0x80000000  // set start variable
	add t0, t1, a0	   // add cache size to start
	// align to 32 bytes by clearing bottom 5 bits
	srl	t0, t0, 5
	sll t0, t0, 5
	addiu t0, t0, -32
	
	move t4, t1
1:
	// update tag ram from ErrCtl
	cache Index_Store_Tag_D, 0(t4)
	bne t0,t4, 1b
	addi t4, t4, 32

	move t4, t1
	move t2, zero
	move t3, zero
1:
	cache Index_Load_Tag_D, 0(t4)
	ehb
	mfc0 t2, C0_DTAGLO
	or t3, t3, t2

	bne t0,t4, 1b
	addi t4, t4, 32

	// end test with return =1 if doesn't matched
	beq zero, t3, 3f
	nop
	
2:
	li v0, 1
	jr ra
	nop

3:
	// start w1 test
	li t2, 0xFFFFFFFF
	
	mtc0 t2, C0_DTAGLO
	move t4, t1
1:
	// update tag ram from ErrCtl
	cache Index_Store_Tag_D, 0(t4)

	bne t0,t4, 1b
	addi t4, t4, 32
	
	move t2, zero
	li t3, 0xffffffff
	move t4, t1
1:
	cache Index_Load_Tag_D, 0(t4)
	ehb
	mfc0 t2, C0_DTAGLO
	and t3, t3, t2

	bne t0,t4, 1b
	addi t4, t4, 32

	// end test with return =2 if doesn't matched
	li t2, 0xFFFFF0E0
	beq t2, t3, DCacheMScanData
	nop
2:
	li v0, 2
	jr ra
	nop

DCacheMScanData:
	// when reach here, pass tag ram mscan test
	// start data ram mscan test

	// prepare to test the data array, w0
	// ErrCtl[PE]=0,ErrCtl[PO]=1,ErrCtl[WST]=1, ErrCtl[PCO]=1
	// parity disable, parity overwrite, test mode enable, overwrite precode
	li t2, 0x68000000
	
	mtc0 t2, C0_ERRCTL
	mtc0 zero, C0_DDATALO
	move t4, t1
1:
	// update data ram from ErrCtl & DDatalo
	cache 0xD, 0(t4)
	cache 0xD, 4(t4)
	cache 0xD, 8(t4)
	cache 0xD, 12(t4)
	cache 0xD, 16(t4)
	cache 0xD, 20(t4)
	cache 0xD, 24(t4)
	cache 0xD, 28(t4)

	bne t0,t4, 1b
	addi t4, t4, 32

	// start r0 test
	// prepare loop variable, 32 byte alignment, t1 is loop variable
	// result store in t3

	// switch to non test mode
	li t2, 0x48000000
	mtc0 t2, C0_ERRCTL

	move t2, zero
	move t3, zero
	move t4, zero

	// mask for PCI, PI bits
	li t5, 0x7FFF0
	move t6, t1
1:
	cache Index_Load_Tag_D, 0(t6)
	ehb
	mfc0 t2, C0_DDATALO
	or t3, t3, t2

	cache Index_Load_Tag_D, 4(t6)
	ehb
	mfc0 t2, C0_DDATALO
	or t3, t3, t2

	cache Index_Load_Tag_D, 8(t6)
	ehb
	mfc0 t2, C0_DDATALO
	or t3, t3, t2

	cache Index_Load_Tag_D, 12(t6)
	ehb
	mfc0 t2, C0_DDATALO
	or t3, t3, t2

	cache Index_Load_Tag_D, 16(t6)
	ehb
	mfc0 t2, C0_DDATALO
	or t3, t3, t2

	cache Index_Load_Tag_D, 20(t6)
	ehb
	mfc0 t2, C0_DDATALO
	or t3, t3, t2

	cache Index_Load_Tag_D, 24(t6)
	ehb
	mfc0 t2, C0_DDATALO
	or t3, t3, t2

	cache Index_Load_Tag_D, 28(t6)
	ehb
	mfc0 t2, C0_DDATALO
	or t3, t3, t2

	mfc0 t2, C0_ERRCTL
	and t2, t2, t5
	or t4, t4, t2

	bne t0,t6, 1b
	addi t6, t6, 32

	// end test with return =3 if doesn't matched
	beq zero, t3, 3f
	nop
	
2:
	li v0, 3
	jr ra
	nop

3:
	// end test with return =4 if PCI/PI doesn't matched
	beq zero, t4, 5f
	nop

4:
	li v0, 4
	jr ra
	nop

5:

	// start w1 test
	// prepare to test the tag array, w0
	// ErrCtl[PE]=0,ErrCtl[PO]=1,ErrCtl[WST]=1, ErrCtl[PCO]=1
	// parity disable, parity overwrite, test mode disable, overwrite precode, set PCI and PI all bits 1
	li t2, 0x6807FFF0
	
	mtc0 t2, C0_ERRCTL
	li t2, 0xFFFFFFFF
	mtc0 t2, C0_DDATALO
	move t4, t1
1:
	// update data ram from ErrCtl & DataLo
	cache 0xD, 0(t4)
	cache 0xD, 4(t4)
	cache 0xD, 8(t4)
	cache 0xD, 12(t4)
	cache 0xD, 16(t4)
	cache 0xD, 20(t4)
	cache 0xD, 24(t4)
	cache 0xD, 28(t4)

	bne t0,t4, 1b
	addi t4, t4, 32

	// start r1 test
	// prepare loop variable, 32 byte alignment, t1 is loop variable
	// result store in t3

	// stay at test mode
	li t2, 0x6807FFF0
	mtc0 t2, C0_ERRCTL
	
	move t2, zero
	li t3, 0xFFFFFFFF
	li t4, 0xFFFFFFFF
	
	// mask for PCI, PI bits
	li t5, 0x7FFF0
	move t6, t1
1:
	

	cache Index_Load_Tag_D, 0(t6)
	ehb
	mfc0 t2, C0_DDATALO
	and t3, t3, t2

	cache Index_Load_Tag_D, 4(t6)
	ehb
	mfc0 t2, C0_DDATALO
	and t3, t3, t2

	cache Index_Load_Tag_D, 8(t6)
	ehb
	mfc0 t2, C0_DDATALO
	and t3, t3, t2

	cache Index_Load_Tag_D, 12(t6)
	ehb
	mfc0 t2, C0_DDATALO
	and t3, t3, t2

	cache Index_Load_Tag_D, 16(t6)
	mfc0 t2, C0_DDATALO
	and t3, t3, t2

	cache Index_Load_Tag_D, 20(t6)
	ehb
	mfc0 t2, C0_DDATALO
	and t3, t3, t2

	cache Index_Load_Tag_D, 24(t6)
	ehb
	mfc0 t2, C0_DDATALO
	and t3, t3, t2

	cache Index_Load_Tag_D, 28(t6)
	ehb
	mfc0 t2, C0_DDATALO
	and t3, t3, t2

	
	mfc0 t2, C0_ERRCTL
	and t2, t2, t5
	and t4, t4, t2

	bne t0,t6, 1b
	addi t6, t6, 32


	// end test with return =5 if doesn't matched
	li t2, 0xFFFFFFFF
	beq t2, t3, 3f
	nop
	
2:
	li v0, 5
	jr ra
	nop

3:

	beq t5, t4, DCacheMScanPass
	nop

4:
	li v0, 6
	jr ra
	nop

DCacheMScanPass:

	// return test with value 0 if no problem
	li v0, 0
	jr ra
	nop

	.set    reorder
END(DCacheMScan)







